package nsf;
import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import org.sanskritlibrary.IOAssistance;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.sanskritlibrary.SAXParserHandler;
import org.sanskritlibrary.SAXParser;

public class CatalogConversion implements SAXParserHandler {
    private SAXParser saxParser;
    private String catalogs;
    private String catalogsV2;
    private BufferedWriter bw;
    private CatalogInfo currentCatalog;
    private Locator locator;
    private StringBuilder currentBuffer = new StringBuilder();
    private Hashtable<String, String> textMappings;
    private Hashtable<String, Boolean> textPrefixes = new Hashtable<String, Boolean>();
    private Hashtable<String, String> prefixMapping = new Hashtable<String, String>();

    public CatalogConversion(String catalogs, String catalogsV2, Hashtable<String, String> textMappings) {
        saxParser = new SAXParser(this);
        this.catalogs = catalogs;
        this.catalogsV2 = catalogsV2;
        this.textMappings = textMappings;

        prefixMapping.put("dur", "Miscellaneous");
        prefixMapping.put("BhG", "BhagavadGita");
        prefixMapping.put("Bag", "BhagavadGita");
        prefixMapping.put("_mi", "Unknown");
        prefixMapping.put("_un", "Unknown");
        prefixMapping.put("MBh", "Mahabharata");
        prefixMapping.put("Siv", "SivaSutras");
        prefixMapping.put("sap", "Miscellaneous");
        prefixMapping.put("VSN", "Miscellaneous");
        prefixMapping.put("azw", "Miscellaneous");
        prefixMapping.put("BhP", "Puranas");
        prefixMapping.put("cat", "Miscellaneous");
        prefixMapping.put("viz", "Miscellaneous");
        prefixMapping.put("Har", "Miscellaneous");
    }

    public void parse () throws Exception {
        bw = IOAssistance.openWriter(catalogsV2);
        saxParser.parse(catalogs);
        bw.close();
        Enumeration<String> en = textPrefixes.keys();
        while (en.hasMoreElements()) {
            System.out.println(en.nextElement());
        }
    }

    public void warning(SAXParseException exception) throws SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void error(SAXParseException exception) throws SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void fatalError(SAXParseException exception) throws SAXException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void setDocumentLocator(Locator locator) {
        this.locator = locator;
    }

    public void startDocument() throws SAXException {
    }

    public void endDocument() throws SAXException {
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException {
    }

    public void endPrefixMapping(String prefix) throws SAXException {
    }

    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        if (qName.equals("manuscripts")) {
            try {
                bw.write("<manuscripts>\n");
            } catch (Exception ex) {
                throw new SAXException(ex.getMessage());
            }
        }
        else if (qName.equals("manuscript")) {
            currentCatalog = new CatalogInfo();
            currentCatalog.id = atts.getValue("id");
        }

        currentBuffer.setLength(0);
    }

    public void endElement(String uri, String localName, String qName) throws SAXException {
        if (qName.equals("manuscripts")) {
            try {
                bw.write("</manuscripts>\n");
            } catch (Exception ex) {
                throw new SAXException(ex.getMessage());
            }
        }
        else if (qName.equals("manuscript")) {
            currentBuffer.setLength(0);
            try {
                bw.write(currentCatalog.toString());
            } catch (Exception ex) {
                throw new SAXException(ex.getMessage());
            }
        } else {
            String buffer = currentBuffer.toString().trim();
            currentBuffer.setLength(0);
            if (qName.equals("directoryName")) {
                currentCatalog.directoryName = buffer;
            } else if (qName.equals("catalogName")) {
                currentCatalog.catalogName = buffer;
            } else if (qName.equals("numLeaves")) {
                currentCatalog.numLeaves = buffer;
            } else if (qName.equals("numFiles")) {
                currentCatalog.numFiles = buffer;
            } else if (qName.equals("title")) {
                currentCatalog.title = buffer;
            } else if (qName.equals("subtitle")) {
                currentCatalog.subtitle = buffer;
            } else if (qName.equals("textType")) {
                textPrefixes.put(buffer.substring(0, 3), true);
                currentCatalog.texts.add(buffer);
            }
        }
    }

    public void characters(char[] ch, int start, int length) throws SAXException {
        if (currentBuffer != null) {
            currentBuffer.append(new String(ch, start, length));
        }
    }

    public class CatalogInfo {

        String id;
        String directoryName;
        ArrayList<String> texts = new ArrayList<String>();
        String catalogName;
        String numLeaves;
        String numFiles;
        String title;
        String subtitle;

        @Override
        public String toString() {
            ArrayList<String> text = new ArrayList<String>();
            ArrayList<String> supp = new ArrayList<String>();
            StringBuilder sb = new StringBuilder();
            sb.append(String.format("<manuscript id='%s' type='%s'>\n", id, prefixMapping.get(texts.get(0).substring(0,3))));
            sb.append(String.format("\t<directoryName>%s</directoryName>\n", directoryName));
            for (String s : texts) {
                if (s.endsWith("Supp")) {
                    supp.add(s);
                } else {
                    text.add(s);
                }
            }

            if (supp.size() == 0) {         // No supplements so no tabs needed
                for (String tx : text) {
                    String mtext = textMappings.get(tx);
                    if (mtext == null) {
                        mtext = tx;
                        System.out.format("%s has no mapping in config.xml\n", tx);
                    }
                    sb.append(String.format("\t<textRef>%s</textRef>\n", mtext));
                }
            } else {            // create Text and Supp tabs
                sb.append("\t<tabs>\n");
                sb.append("\t\t<tab name='Text'>\n");
                // Write stuff that will appear in Text tab
                for (String tx : text) {
                    String mtext = textMappings.get(tx);
                    if (mtext == null) {
                        mtext = tx;
                        System.out.format("%s has no mapping in config.xml\n", tx);
                    }
                    sb.append(String.format("\t\t\t<textRef>%s</textRef>\n", tx));
                }
                sb.append("\t\t</tab>\n");

                sb.append("\t\t<tab name='Supplement'>\n");
                // Write stuff that will appear in Supp tab
                for (String sp : supp) {
                    sb.append(String.format("\t\t\t<textRef>%s</textRef>\n", sp));
                }
                sb.append("\t\t</tab>\n");

                sb.append("\t</tabs>\n");
            }
            sb.append(String.format("\t<catalogName>%s</catalogName>\n", catalogName));
            sb.append(String.format("\t<numLeaves>%s</numLeaves>\n", numLeaves));
            sb.append(String.format("\t<numFiles>%s</numFiles>\n", numFiles));
            sb.append(String.format("\t<title>%s</title>\n", title));
            sb.append(String.format("\t<subtitle>%s</subtitle>\n", subtitle));
            sb.append("</manuscript>\n");

            return sb.toString();
        }
    }
}
