package nsf;

import java.io.*;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Collections;
import java.nio.charset.Charset;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.RawDecoder;
import org.sanskritlibrary.RawEncoder;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.RTF2;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.webservice.WebServices;
import org.sanskritlibrary.TextFormatter;

public class FreundTextConversion
{
        public static VedicText[] texts = new VedicText[]
        {
            // Darshanas
            new VedicText("Darshanas", "karma_mimansa", "txt"), // 0
            new VedicText("Darshanas", "karma_mimansa_sankarshana", "txt"), // 1
            new VedicText("Darshanas", "karma_mimansa_sankarshana_2", "txt"), // 2
            new VedicText("Darshanas", "Nyaya", "txt"),  // 3
            new VedicText("Darshanas", "samkhya", "txt"), // 4
            new VedicText("Darshanas", "vaisheshika", "txt"), // 5
            new VedicText("Darshanas", "vedanta", "txt"), // 6

            // Ayurveda
            new VedicText("Ayurveda", "bhava_prakasha_samhita", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Ayurveda", "bhela_samhita", "txt"),
            new VedicText("Ayurveda", "harita_samhita", "rtf", new String[] {"mahoregvedic", "mahoregatharva"}),
            new VedicText("Ayurveda", "kashyapa_samhita", "rtf", new String[] {"vedic", "mahoregvedic", "mahoregatharva"}),
            new VedicText("Ayurveda", "madhava_nidhana_samhita", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Ayurveda", "sharngadhara_samhita", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Ayurveda", "sushruta", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Ayurveda", "vagbhata_samhita", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Aranyakas
            new VedicText("Brahmana Loop/Aranyakas", "chandogya aranyaka", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Aranyakas", "jaiminiya aranyaka", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Aranyakas", "katha aranyaka", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Aranyakas", "maitrayani aranyaka", "rtf", new String[] {"vedic", "mahoregyajur", "mahoregvedic", "mahoregshukla", "mahoregrik"}),
            new VedicText("Brahmana Loop/Aranyakas", "taittiriya aranyaka", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Brahmanas
            new VedicText("Brahmana Loop/Brahmanas", "aitareya brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "chandogya brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "jaimini talavakara brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "jaiminiya upanishad brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "kathaka brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "taittiriya brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Brahmanas", "vamsha brahmana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Upanishads
            new VedicText("Brahmana Loop/Upanishads", "15 Upanishads starting with Isha", "rtf", new String[] {"vedictrans97", "mahoregyajur", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Upanishads", "108 Upanishads", "txt"),
            new VedicText("Brahmana Loop/Upanishads", "brihadaranyaka upanishad", "txt"),
            new VedicText("Brahmana Loop/Upanishads", "jabala upanishad", "txt"),
            new VedicText("Brahmana Loop/Upanishads", "kaushitaki brahmana upanishad", "rtf", new String[] {"vedic97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Upanishads", "mahanarayana upanishad", "txt"),
            new VedicText("Brahmana Loop/Upanishads", "pingala upanishad", "txt"),
            new VedicText("Brahmana Loop/Upanishads", "rik veda upanishads", "rtf", new String[] {"vedic97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Upanishads", "samaveda upanishads", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Brahmana Loop/Upanishads", "tarasara upanishad", "rtf", new String[] {"mahoregyajur", "mahoregvedic"}),

            // Charaka
            new VedicText("Charaka", "Charaka 01 Sutra Sthana", "txt"),
            new VedicText("Charaka", "Charaka 02 Nidana Sthana", "txt"),
            new VedicText("Charaka", "Charaka 03 Vimana Sthana", "txt"),
            new VedicText("Charaka", "Charaka 04 Sharira Sthana", "txt"),
            new VedicText("Charaka", "Charaka 05 Indriya Sthana", "txt"),
            new VedicText("Charaka", "Charaka 06A Chikitsa Sthana", "txt"),
            new VedicText("Charaka", "Charaka 06B Chikitsa Sthana", "txt"),
            new VedicText("Charaka", "Charaka 06C Chikitsa Sthana", "txt"),
            new VedicText("Charaka", "Charaka 07 Kalpa Sthana", "txt"),
            new VedicText("Charaka", "Charaka 08 Siddhi Sthana", "txt"),

            // Itihasa
            new VedicText("Itihasa", "mahabharata 1 2 3", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Itihasa", "mahabharata 4 5 6 7", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Itihasa", "mahabharata 8 9 10 11 12", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Itihasa", "mahabharata 13 14 15 16 17 18", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Itihasa", "ramayana", "rtf", new String[] {"mahoregvedic"}),

            // Pratishakhya
            new VedicText("Pratishakhya", "atharva veda pratishakhya chaturadhyayi", "txt"),
            new VedicText("Pratishakhya", "atharva veda pratishakhya", "rtf", new String[] {"mahoreg"}),
            new VedicText("Pratishakhya", "rik veda pratishakhya", "txt"),
            new VedicText("Pratishakhya", "sama veda pratishakhya", "rtf", new String[] {"mahoregyajur", "mahoregvedic"}),
            new VedicText("Pratishakhya", "shukla yajur veda pratishakhya", "txt"),
            new VedicText("Pratishakhya", "taittiriya pratishakhya", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Purana
            new VedicText("Purana", "agni purana", "rtf", new String[] {"mahoregvedic", "mahoregshukla"}),
            new VedicText("Purana", "bhagavata purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "brahma purana", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Purana", "brahmanda purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "garuda purana", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Purana", "kurma purana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Purana", "linga purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "markandeya purana", "rtf", new String[] {"vedic97", "vedic", "mahoregvedic"}),
            new VedicText("Purana", "matsya purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "narada purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "padma purana I", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "padma purana II", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "padma purana III", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "shiva purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Purana", "skanda purana 25 chapters", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Purana", "vamana purana", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Purana", "vishnu purana", "rtf", new String[] {"mahoregvedic"}),

            // Smriti
            new VedicText("Smriti", "brihaspati smriti", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Smriti", "smriti complete", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Smriti", "vasishtha smriti", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Stotras
            new VedicText("Stotras", "acyutastaka", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "durga sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "durga saptashati", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "durgashtaka", "txt"),
            new VedicText("Stotras", "ganapati sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "ganga sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "gangashtaka", "txt"),
            new VedicText("Stotras", "gayatri ashtaka", "txt"),
            new VedicText("Stotras", "gayatri sahasranama", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Stotras", "guru gita", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "hanuman stavan", "rtf", new String[] {"vedic97", "mahoregvedic"}),
            new VedicText("Stotras", "lakshmi sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "lalita sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "lingashtaka", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "mahalakshmi ashtakam", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "nirvana dashakam", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "rama raksha stotram", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "rama sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "Rina Vimocana nirisimha stotra", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Stotras", "sankastha ganesha stotram", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "sarasvati ashtakam", "txt"),
            new VedicText("Stotras", "sarasvati sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "shiva sahasranama", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Stotras", "shri natesha sahasranama", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Stotras", "surya mandala ashtakam", "txt"),
            new VedicText("Stotras", "surya sahasranama", "txt"),
            new VedicText("Stotras", "vishnu astottara shatanamavali", "txt"),
            new VedicText("Stotras", "vishnu sahasranama", "rtf", new String[] {"mahoregvedic"}),

            // Upapurana
            new VedicText("Upapurana", "kapila purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "nandi purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "narasimha purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "nilamata upapurana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "parashara upapurana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "samba purana", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upapurana", "saura purana", "txt"),
            new VedicText("Upapurana", "vishnudharma", "rtf", new String[] {"mahoregvedic"}),

            // Upasmriti
            new VedicText("Upasmriti", "devala smriti", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upasmriti", "kashyapa smriti", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Upasmriti", "narada smriti", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upasmriti", "vishvamitra smriti", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Upasmriti", "yama smriti", "rtf", new String[] {"vedic", "mahoregvedic"}),

            // Dhanur Veda
            new VedicText("Upaveda/Dhanur Veda", "niti prakashika", "txt"),
            new VedicText("Upaveda/Dhanur Veda", "shiva dhanur veda", "txt"),
            new VedicText("Upaveda/Dhanur Veda", "vasishtha dhanur veda", "txt"),

            // Gandharva Veda
            new VedicText("Upaveda/Gandharva Veda", "natya shastra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Gandharva Veda", "sangirta ratnakara", "txt"),

            // Sthapatya Veda
            new VedicText("Upaveda/Sthapatya Veda", "Kamikagama", "rtf", new String[] {"vedictrans97", "vedic97", "mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "kashyapa shilpa shastra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "kiranagama", "txt"),
            new VedicText("Upaveda/Sthapatya Veda", "manasara", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "manushyalaya chandrika", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "samaranganasutradhara", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "vastusutra upanishad", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "vishvakarma vastu shastra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Upaveda/Sthapatya Veda", "yogajagama", "rtf", new String[] {"mahoregvedic"}),

            // Vedangas
            new VedicText("Vedangas", "complete_shiksha_book_sequence", "rtf", new String[] {"mahoregyajur", "mahoregvedic", "mahoregshukla", "mahoregrik"}),
            new VedicText("Vedangas", "dantyoshthya vidhi", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas", "kanwa shiksha", "rtf", new String[] {"mahoregvedic", "mahoregshukla"}),
            new VedicText("Vedangas", "kaundinya shiksha", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas", "madhyandina shiksha", "rtf", new String[] {"mahoregyajur"}),
            new VedicText("Vedangas", "padacandrika", "rtf", new String[] {"mahoregyajur", "mahoregvedic"}),
            new VedicText("Vedangas", "padakarikaratnamala", "rtf", new String[] {"mahoregvedic"}),

            // Jyotish
            new VedicText("Vedangas/Jyotish", "lagadha vedanga jyotish", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Jyotish", "muhurta chintamani", "txt"),
            new VedicText("Vedangas/Jyotish", "parashara hora shastra", "txt"),
            new VedicText("Vedangas/Jyotish", "Shatpanchashika", "txt"),
            new VedicText("Vedangas/Jyotish", "surya siddhanta", "txt"),

            // Dharma Sutra
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "apastamba dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "baudhayana dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "gautama dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "vaikhanasa dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "vasishtha dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Dharma Sutra", "vishnu dharma sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),

            // Grihya Sutra
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "agniveshya grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "apastamba grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "ashvalayana grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "bharadvaja grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "bodhayana grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "drahyayana grihya sutra", "rtf", new String[] {"vedic97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "gobhila grihya sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "hiranyakeshi grihya sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "jaimini grihya sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "kathaka grihya sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "kaushika grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "kaushitaka grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "kauthuma grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "manava grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "paraskara grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "shankhayana grihya sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "vadhula grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "vaikhanasa grihya sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Grihya Sutra", "varaha grihya sutra", "txt"),

            // Shrauta Sutra
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "apastamba shrauta sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "ashvalayana shrauta sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "baudhayana shrauta sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "bharadvaja shrauta sutra", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "drahyayana shrauta sutra", "rtf", new String[] {"vedictrans97", "mahoregyajur", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "jaimini shrauta sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "katyayana shrauta sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "kshudra sutra", "rtf", new String[] {"mahoregyajur", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "latyayana shrauta sutra", "rtf", new String[] {"mahoregyajur", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "manava shrauta sutra", "rtf", new String[] {"vedic", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "mashaka shrauta sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "nidana shrauta sutra", "rtf", new String[] {"mahoregyajur", "mahoregvedic", "mahoregsama", "mahoregshukla", "mahoregrik"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "nidana sutra with accents", "rtf", new String[] {"mahoregyajur", "mahoregvedic", "mahoregsama", "mahoregshukla", "mahoregrik"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "panchavidha sutra", "rtf", new String[] {"mahoregyajur", "mahoregsama"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "shankhayana shrauta sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "upagrantha sutra", "rtf", new String[] {"mahoregvedic", "mahoreg"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "vaikhanasa shrauta sutra", "rtf", new String[] {"vedic97", "mahoregyajur", "mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "vaitana shrauta sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shrauta Sutra", "varaha shrauta sutra", "rtf", new String[] {"vedic97", "mahoregvedic"}),

            // Shulba Sutra
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "Apastamba Shulba Sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "Baudhayana Shulba Sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "Hiranyakeshi Shulba Sutra", "rtf", new String[] {"vedictrans97", "mahoregyajur"}),
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "Katyayana Shulba Sutra", "txt"),
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "Maitrayaniya Shulba Sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Kalpa/Shulba Sutra", "manava shulba sutra", "txt"),

            // Nirukta
            new VedicText("Vedangas/Nirukta", "nirukta mahoregrik", "txt"),

            // Vyakarana
            new VedicText("Vedangas/Vyakarana", "dhatupatha", "rtf", new String[] {"vedictrans97", "mahoregvedic"}),
            new VedicText("Vedangas/Vyakarana", "ganapatha", "txt"),
            new VedicText("Vedangas/Vyakarana", "linganushasanam", "rtf", new String[] {"vedic97", "vedcon97", "vedpunc97", "vedi97", "vedixymbol97", "vedik97"}),
            new VedicText("Vedangas/Vyakarana", "phit sutra", "txt"),
            new VedicText("Vedangas/Vyakarana", "unadi sutra", "rtf", new String[] {"mahoregvedic"}),
            new VedicText("Vedangas/Vyakarana", "vyakarana", "txt"),

            // Yoga texts
            new VedicText("Yoga texts", "bhagavad gita", "rtf", new String[] {"vedic97", "mahoregvedic"}),
            new VedicText("Yoga texts", "shiva samhita", "txt"),
            new VedicText("Yoga texts", "shiva sutra", "txt"),
            new VedicText("Yoga texts", "vigyana bhairava", "txt"),
        };

    static Main main = new Main();
    static Hashtable<String, FSM[]> fmsh = new Hashtable<String, FSM[]>();
    static Hashtable<String, Boolean> ignore = new Hashtable<String, Boolean>();
    //static String htmlDirectory = Main.psDirectory + "\\Vedatype\\html";
    static String transcodersDirectory = "C://Users//ralph//My Documents//PeterScharf//SanskritLibrary/data/transcoders";
    public static void transcodeTexts() throws Exception
    {

        Hashtable<String, Boolean> stoplist = new Hashtable<String, Boolean>();
        stoplist.put("mahoregatharva", true);
        stoplist.put("mahoregyajur", true);
        stoplist.put("mahoregshukla", true);
        stoplist.put("mahoregrik", true);
        stoplist.put("mahoregsama", true);
        stoplist.put("mahoreg", true);
        stoplist.put("vedcon97", true);
        stoplist.put("vedpunc97", true);
        stoplist.put("vedi97", true);
        stoplist.put("vedixsymbol97", true);
        stoplist.put("vedik97", true);

        boolean doWork = true;

        if (!doWork)
            return;

        new TransformMap("*:vedic97->vedatype", main, fmsh);
        new TransformMap("*:vedatype->deva", main, fmsh);
        new TransformMap("*:vedic97->deva", main, fmsh);
        BufferedWriter errorReport = IOAssistance.openWriter(String.format("%s/errors.txt", VedicText.vedicLiteratureOutputDirectory));
        boolean doit = false;
        for (int i = 0; i < texts.length; i++)
        {
            VedicText text = texts[i];
            if (!text.type.equals("rtf"))
                continue;
            
            boolean bContinue = true;
            for (String font : text.fonts)
            {
                if (stoplist.get(font) !=  null)        // uses a font with non-vedic97 characters
                    bContinue = false;
            }

            if (bContinue)
                continue;

            System.out.format("*********%s\n", text.filename);
/*
            int errors = transcodeText(text);
            errorReport.write(text.getInputFileName());
            if (errors > 0)
                errorReport.write(String.format(" %d errors\n", errors));
            else
                errorReport.write("\n");
 */
        }
        errorReport.close();
    }


    // http://kevincarmody.com/software/aksharabridge.html
    public static void aksharaBridge(String filename) throws Exception
    {
        String vedic97File = String.format("%s.txt2", filename);
        String vedapadFile = String.format("%s.vp", filename);
        String unicodeFile = String.format("%s.uni", filename);
        String vedapadErrorFile = String.format("%s_vp.err", filename);
        String unicodeErrorFile = String.format("%s_uni.err", filename);
        String cmdline = String.format("C:\\Program Files\\Akshara Bridge\\AksharaBridgeCmd.exe /t=encode2key /s=vedic97 /i=\"%s\" /o=\"%s\" /e=\"%s\"",
                                    vedic97File, vedapadFile, vedapadErrorFile);
        Runtime rt = Runtime.getRuntime();
        Process p = rt.exec(cmdline);
        if (p.waitFor() != 0)
            throw new Exception(String.format("%s failed", cmdline));

        cmdline = String.format("C:\\Program Files\\Akshara Bridge\\AksharaBridgeCmd.exe /t=key2unidev /s=vedapad /i=\"%s\" /o=\"%s\" /e=\"%s\"",
                                    vedapadFile, unicodeFile, unicodeErrorFile);
        p = rt.exec(cmdline);

        // need to do this or else stdout buffer might get full?
        InputStream is = p.getInputStream();
        int ch = 0;
        int n = 0;
        while ((ch = is.read()) != -1)
        {
            n++;
            if (n == 80)
            {
                n = 0;
                System.out.println();
            }
            System.out.print((char)ch);
        }
        System.out.println();

        if (p.waitFor() != 0)
            throw new Exception(String.format("%s failed\n", cmdline));

        System.out.println("Leaving aksharaBridge");
    }

    public static int transcodeText(VedicText text) throws Exception
    {
        if (text.type.equals("txt"))
            return transcodeTxtFile(text, true);
        else
            return transcodeRtfFile(text);
    }

    private static int transcodeTxtFile(VedicText text, boolean copy) throws Exception
    {
        try
        {
            String line = null;
            BufferedReader br = null;
            int lineno = 0;
            String filetxt2 = text.getInputFileName() + ".txt2";
            if (!IOAssistance.fileExists(filetxt2))
            {
                String origfile = text.getInputFilePath();
                br = IOAssistance.openReader(origfile, new RawDecoder());
                BufferedWriter bw = IOAssistance.openWriter(filetxt2, new RawEncoder());
                while ((line = br.readLine()) != null)
                {
                    Regex re = new Regex(line, "(.*?(?:\\s*$|\\d+-?\\d*))\\s*");
                    boolean bfound = false;
                    while (re.find())
                    {
                        String verse = re.getGroup(1);
                        if (verse.length() > 0)
                        {
                            bfound = true;
                            bw.write(re.getGroup(1));
                            bw.write("\n");
                        }
                    }
                    if (!bfound)
                    {
                        bw.write(line);
                        bw.write("\n");
                    }
                }

                br.close();
                bw.close();
            }

            if (!IOAssistance.fileExists(text.getInputFileName() + ".vp"))
                aksharaBridge(text.getInputFileName());

            String outfileVedatype = text.getInputFileName() + ".vp2";
            String outfileDeva = text.getInputFileName() + ".uni2";

            String map1 = "*:vedic97->vedatype";
            String map2 = "*:vedatype->deva";
            TransformMap tmfv = new TransformMap(map1, main, fmsh);
            TransformMap tmfdeva = new TransformMap(map2, main, fmsh);
            br = IOAssistance.openReader(filetxt2, new RawDecoder());
            BufferedWriter bwVedatype = IOAssistance.openWriter(outfileVedatype);
            BufferedWriter bwDeva = IOAssistance.openWriter(outfileDeva, Charset.forName("UTF-16"));
            line = null;
            while ((line = br.readLine()) != null)
            {
                lineno++;
                line = WebServices.transformString(line, tmfv);  // convert Vedic 97 to vedatype via SLP1
                bwVedatype.write(line);
                bwVedatype.write("\r\n");

                line = WebServices.transformString(line, tmfdeva); // convert Vedatype to deva via SLP1
                bwDeva.write(line);
                bwDeva.write("\r\n");

            }
            bwVedatype.close();
            bwDeva.close();
            br.close();

            Hashtable<Integer, Boolean> h = new Hashtable<Integer, Boolean>();
            compareOutput(text, "vp", h);
            compareOutput(text, "uni", h);
            if (h.size() > 0)
                displaySummary(text, h);

            if (copy)
                text.copy("uni2", "UTF-16");

            return h.size();
        } catch (Exception ex)
        {
            ex.printStackTrace();
            throw ex;
        }

    }

    private static int transcodeRtfFile(VedicText text) throws Exception
    {
/*
        boolean skip = true;
        if (skip)
            return 0;
*/
        RTF2 rtf = new RTF2(text.getInputFilePath(), "windows");
        rtf.setUnicodeFontFormat("{\\f%d\\fbidi \\froman\\fcharset0\\fprq2{\\*\\panose 02040503050203030202}Praja;}");
        rtf.accumulateToEOL(true);
        for (String fontname : text.fonts)
        {
            String map = String.format("*:%s->deva", fontname);
            rtf.addMap(fontname, new TransformMap(map, main, fmsh));
        }

        String extractionRawFilePath = String.format("%s.txt2", text.getInputFileName());
        rtf.setExtractionRaw(IOAssistance.openWriter(extractionRawFilePath, new RawEncoder(Charset.forName("UTF-8"), 1, 1)));

        String outputFile = text.getOutputFilePath();
        rtf.setOutputFile(IOAssistance.openWriter(outputFile));

        rtf.parse();
        rtf.close();

        return transcodeTxtFile(text, false);
    }

    public static void displaySummary(VedicText text, Hashtable<Integer, Boolean> h)
            throws Exception
    {
        String filename = text.getInputFileName();
        int len = h.size();
        if (len > 0)
        {
            String filePathVedic97 = String.format("%s.txt2", filename, filename);
            String filePathKevinVedatype = String.format("%s.vp", filename);
            String filePathRalphVedatype = String.format("%s.vp2", filename);
            String filePathKevinUnicode = String.format("%s.uni", filename);
            String filePathRalphUnicode = String.format("%s.uni2", filename);

            BufferedReader brVedic97 = IOAssistance.openReader(filePathVedic97, new RawDecoder());
            BufferedReader brKevinVedatype = IOAssistance.openReader(filePathKevinVedatype);
            BufferedReader brRalphVedatype = IOAssistance.openReader(filePathRalphVedatype);
            BufferedReader brKevinUnicode = IOAssistance.openReader(filePathKevinUnicode, "UTF-16");
            BufferedReader brRalphUnicode = IOAssistance.openReader(filePathRalphUnicode, "UTF-16");

            String fileSummary = String.format("%s.htm", text.getOutputFileName());
            //String fileSummaryVedic97 = String.format("%s\\%sVedic97.htm", htmlDirectory, text.filename);

            BufferedWriter bwSummary = IOAssistance.openWriter(fileSummary, Charset.forName("UTF-16"));
            //BufferedWriter bwSummaryVedic97 = IOAssistance.openWriter(fileSummaryVedic97, new RawEncoder());
            bwSummary.write("<html><head><style type='text/css'>.f8 {font-family:Vedic97; font-size:32px}.v {font-size: 20px} .d {font-size:28px;}</style></head><body><div>\n");
            //bwSummaryVedic97.write("<html><head><style type='text/css'>.f8 {font-family:Vedic97; font-size:36px;}</style></head><body><div>\n");
            int lineno = 0;
            String lineVedic97 = null;
            String lineKevinVedatype = null;
            String lineRalphVedatype = null;
            String lineKevinUnicode = null;
            String lineRalphUnicode = null;

            ArrayList<Integer> a = new ArrayList<Integer>();
            Enumeration<Integer> e = h.keys();
            while (e.hasMoreElements())
            {
                a.add(e.nextElement());
            }
            Collections.sort(a);
            for (int i = 0; i < len; i++)
            {
                int n = a.get(i);
                while (lineno != n)
                {
                    lineno++;
                    lineVedic97 = brVedic97.readLine();
                    lineKevinVedatype = brKevinVedatype.readLine();
                    lineRalphVedatype = brRalphVedatype.readLine();
                    lineKevinUnicode = brKevinUnicode.readLine();
                    lineRalphUnicode = brRalphUnicode.readLine();
                }

                bwSummary.write(Integer.toString(lineno));

                //bwSummaryVedic97.write(Integer.toString(lineno));
                //bwSummaryVedic97.write(String.format("<p class='f8'>%s</p>", lineVedic97));
                //bwSummaryVedic97.write("<hr/>");

                bwSummary.write(String.format("<p class='f8'>%s</p>", lineVedic97));
                int[] range = findFirstDifference(lineKevinVedatype, lineRalphVedatype);
                int first = range[0];
                int lastKevin = range[1];
                int lastRalph = range[2];
                bwSummary.write(String.format("<p class='v'>%s</p>", TextFormatter.charToEntity(markDifference(lineKevinVedatype, first, lastKevin))));
                bwSummary.write(String.format("<p class='v'>%s</p>", TextFormatter.charToEntity(markDifference(lineRalphVedatype, first, lastRalph))));

                range = findFirstDifference(lineKevinUnicode, lineRalphUnicode);
                first = range[0];
                lastKevin = range[1];
                lastRalph = range[2];

                bwSummary.write(String.format("<p class='d'>%s</p>", markDifference(lineKevinUnicode, first, lastKevin)));
                bwSummary.write(String.format("<p class='d'>%s</p>", markDifference(lineRalphUnicode, first, lastRalph)));

                bwSummary.write("<hr/>");
            }

            bwSummary.write("</div></body></html>\n");
            //bwSummaryVedic97.write("</div></body></html>\n");

            bwSummary.close();
            //bwSummaryVedic97.close();
            brVedic97.close();
            brKevinVedatype.close();
            brRalphVedatype.close();
            brKevinUnicode.close();
            brRalphUnicode.close();
        }
    }

    public static String markDifference(String line, int first, int last)
    {
        if (first <= last)
        {
            StringBuilder sb = new StringBuilder();
            sb.append(line.substring(0, first));
            sb.append("<span style='color:red'>");
            if (last + 1 > line.length())
                last = line.length()-1;
            sb.append(line.substring(first, last + 1));
            sb.append("</span>");
            sb.append(line.substring(last + 1));
            return sb.toString();
        } else
        {
            return line;
        }
    }

    public static void compareOutput(VedicText text, String type,
            Hashtable<Integer, Boolean> h)
            throws Exception
    {
        String filename = text.getInputFileName();
        String fileKevinOutput = String.format("%s.%s", filename, type);
        if (IOAssistance.fileExists(fileKevinOutput))
        {
            String fileRalphOutput = String.format("%s.%s2", filename, type);
            BufferedReader brKevinOutput = null;
            BufferedReader brRalphOutput = null;
            if (type.equals("vp"))
            {
                brKevinOutput = IOAssistance.openReader(fileKevinOutput);
                brRalphOutput = IOAssistance.openReader(fileRalphOutput);
            } else
            {
                brKevinOutput = IOAssistance.openReader(fileKevinOutput, "UTF-16");
                brRalphOutput = IOAssistance.openReader(fileRalphOutput, "UTF-16");
            }

            String lineKevin = null;
            String lineRalph = null;
            int lineno = 0;
            int errors = 0;
            do
            {
                lineno++;
                lineKevin = brKevinOutput.readLine();
                if (lineKevin == null)
                {
                    break;
                }
                if (type.equals("vp"))
                {
                    lineKevin = lineKevin.replace("[db]", "");
                } else
                {
                    lineKevin = lineKevin.replace("\ua8fb", "");		// get rid of horizontal bar
                }
                lineRalph = brRalphOutput.readLine();
                if (lineRalph == null)
                {
                    break;
                }
                if (!lineKevin.equals(lineRalph))
                {
                    h.put(lineno, true);
                    errors++;
                }
            } while (lineKevin != null && lineRalph != null);

            // System.out.format("Number of errors for %s: %s\n", type, errors);
            brKevinOutput.close();
            brRalphOutput.close();
        }
    }

    static int[] findFirstDifference(String lineKevin, String lineRalph)
    {
        int first = -1;
        int lastRalph = 0;
        int lastKevin = 0;
        int[] range = new int[3];
        int linelen = Math.min(lineKevin.length(), lineRalph.length());
        for (int i = 0; i < linelen; i++)
        {
            if (lineKevin.charAt(i) != lineRalph.charAt(i))
            {
                first = i;
                break;
            }
        }
        if (first == -1) // must be line difference;
        {
            first = linelen - 1;
        }
        // find the last difference
        for (int i = lineKevin.length() - 1, j = lineRalph.length() - 1; j >= 0 && i >= 0; i--, j--)
        {
            if (lineKevin.charAt(i) != lineRalph.charAt(j))
            {
                lastKevin = i;
                lastRalph = j;
                break;
            }
        }

        first = Math.max(0, first - 4);
        lastKevin = Math.min(lineKevin.length(), lastKevin);
        lastRalph = Math.min(lineRalph.length(), lastRalph);

        range[0] = first;
        range[1] = lastKevin;
        range[2] = lastRalph;

        return range;
    }

    public static void displayFontsUsed(VedicText[] texts)
    {
        Hashtable<String, ArrayList<VedicText>> h = new Hashtable<String, ArrayList<VedicText>>();
        int count = 0;
        for (VedicText vt : texts)
        {
            if (vt.type.equals("rtf"))
            {
                ArrayList<String> a = vt.getUsedFonts();
                if (a != null)
                {
                    for (String s : a)
                    {
                        ArrayList<VedicText> list = h.get(s);
                        if (list == null)
                        {
                            list = new ArrayList<VedicText>();
                            System.out.println(s);
                            h.put(s, list);
                        }
                        list.add(vt);
                    }
                }
                else
                    System.out.format("Why is a null for %s", vt.filename);
            }
        }

        Enumeration<String> e = h.keys();
        while (e.hasMoreElements())
        {
            String key = e.nextElement();
            System.out.format("Font: %s\n", key);
            ArrayList<VedicText> a = h.get(key);
            for (VedicText vt : a)
            {
                System.out.format("\t%s\n", vt.filename);
            }
        }
    }

    public static void displayTextFonts(VedicText[] texts)
    {
        Hashtable<String, Boolean> fonts = new Hashtable<String, Boolean>();
        fonts.put("mahoregvedic", true);
        fonts.put("vedic", true);
        fonts.put("vedic97", true);
        fonts.put("mahoreg", true);
        fonts.put("mahoregyajur", true);
        fonts.put("mahoregshukla", true);
        fonts.put("mahoregatharva", true);
        fonts.put("mahoregsama", true);
        fonts.put("mahoregrik", true);
        fonts.put("vedictrans97", true);


        Hashtable<String, ArrayList<VedicText>> h = new Hashtable<String, ArrayList<VedicText>>();
        for (VedicText vt : texts)
        {
            if (vt.type.equals("rtf"))
            {
                ArrayList<String> a = vt.getUsedFonts();
                String delim = "";
                if (a != null)
                {
                    System.out.println(vt.filename);

                    System.out.print("new String[] {");

                    for (String s : a)
                    {
                        if (fonts.get(s) != null)
                        {
                            System.out.format("%s\"%s\"", delim, s);
                            delim = ", ";
                        }
                    }
                    System.out.println("}\n");
                }
                else
                    System.out.format("Why is a null for %s", vt.filename);
            }
        }
    }
}



/*
x mahoregvedic
x vedic
x vedic97
x mahoreg

x mahoregyajur
x mahoregshukla
x mahoregatharva
x mahoregsama
x mahoregrik

x vedictrans97
shep == vedictrans97

Only used by linganushasanam
vedcon97
vedpunc97
vedi97
vedixymbol97
vedik97


Font: vedi97
        linganushasanam

Font: vedixymbol97
        linganushasanam

Font: vedpunc97
        linganushasanam

Font: vedcon97
        linganushasanam

Font: vedik97
        linganushasanam

Font: mahoregvedic
        bhava_prakasha_samhita
        harita_samhita
        kashyapa_samhita
        madhava_nidhana_samhita
        sharngadhara_samhita
        sushruta
        vagbhata_samhita
        chandogya aranyaka
        jaiminiya aranyaka
        katha aranyaka
        maitrayani aranyaka
        taittiriya aranyaka
        aitareya brahmana
        chandogya brahmana
        jaimini talavakara brahmana
        jaiminiya upanishad brahmana
        kathaka brahmana
        taittiriya brahmana
        vamsha brahmana
        15 Upanishads starting with Isha
        kaushitaki brahmana upanishad
        rik veda upanishads
        samaveda upanishads
        tarasara upanishad
        mahabharata 1 2 3
        mahabharata 4 5 6 7
        mahabharata 8 9 10 11 12
        mahabharata 13 14 15 16 17 18
        ramayana
        sama veda pratishakhya
        taittiriya pratishakhya
        agni purana
        bhagavata purana
        brahma purana
        brahmanda purana
        garuda purana
        kurma purana
        linga purana
        markandeya purana
        matsya purana
        narada purana
        padma purana I
        padma purana II
        padma purana III
        shiva purana
        skanda purana 25 chapters
        vamana purana
        vishnu purana
        brihaspati smriti
        smriti complete
        vasishtha smriti
        acyutastaka
        durga sahasranama
        durga saptashati
        ganapati sahasranama
        ganga sahasranama
        gayatri sahasranama
        guru gita
        hanuman stavan
        lakshmi sahasranama
        lalita sahasranama
        lingashtaka
        mahalakshmi ashtakam
        nirvana dashakam
        rama raksha stotram
        rama sahasranama
        Rina Vimocana nirisimha stotra
        sankastha ganesha stotram
        sarasvati sahasranama
        shiva sahasranama
        shri natesha sahasranama
        vishnu sahasranama
        kapila purana
        nandi purana
        narasimha purana
        nilamata upapurana
        parashara upapurana
        samba purana
        vishnudharma
        devala smriti
        kashyapa smriti
        narada smriti
        vishvamitra smriti
        yama smriti
        natya shastra
        Kamikagama
        kashyapa shilpa shastra
        manasara
        manushyalaya chandrika
        samaranganasutradhara
        vastusutra upanishad
        vishvakarma vastu shastra
        yogajagama
        complete_shiksha_book_sequence
        dantyoshthya vidhi
        kanwa shiksha
        kaundinya shiksha
        padacandrika
        padakarikaratnamala
        lagadha vedanga jyotish
        apastamba dharma sutra
        baudhayana dharma sutra
        gautama dharma sutra
        vaikhanasa dharma sutra
        vasishtha dharma sutra
        vishnu dharma sutra
        drahyayana grihya sutra
        gobhila grihya sutra
        hiranyakeshi grihya sutra
        jaimini grihya sutra
        kathaka grihya sutra
        shankhayana grihya sutra
        ashvalayana shrauta sutra
        baudhayana shrauta sutra
        bharadvaja shrauta sutra
        drahyayana shrauta sutra
        katyayana shrauta sutra
        kshudra sutra
        latyayana shrauta sutra
        manava shrauta sutra
        nidana shrauta sutra
        nidana sutra with accents
        upagrantha sutra
        vaikhanasa shrauta sutra
        vaitana shrauta sutra
        varaha shrauta sutra
        Maitrayaniya Shulba Sutra
        dhatupatha
        unadi sutra
        bhagavad gita

Font: mahoregrik
        maitrayani aranyaka
        complete_shiksha_book_sequence
        nidana shrauta sutra
        nidana sutra with accents

Font: mahoreg
        atharva veda pratishakhya
        upagrantha sutra

Font: mahoregyajur
        maitrayani aranyaka
        15 Upanishads starting with Isha
        tarasara upanishad
        sama veda pratishakhya
        complete_shiksha_book_sequence
        madhyandina shiksha
        padacandrika
        drahyayana shrauta sutra
        kshudra sutra
        latyayana shrauta sutra
        nidana shrauta sutra
        nidana sutra with accents
        panchavidha sutra
        vaikhanasa shrauta sutra
        Hiranyakeshi Shulba Sutra

Font: vedic97
        kaushitaki brahmana upanishad
        rik veda upanishads
        markandeya purana
        hanuman stavan
        Kamikagama
        drahyayana grihya sutra
        vaikhanasa shrauta sutra
        varaha shrauta sutra
        linganushasanam
        bhagavad gita

Font: vedictrans97
        vagbhata_samhita
        chandogya aranyaka
        jaiminiya aranyaka
        katha aranyaka
        taittiriya aranyaka
        aitareya brahmana
        chandogya brahmana
        jaimini talavakara brahmana
        jaiminiya upanishad brahmana
        kathaka brahmana
        taittiriya brahmana
        vamsha brahmana
        15 Upanishads starting with Isha
        samaveda upanishads
        taittiriya pratishakhya
        kurma purana
        skanda purana 25 chapters
        vamana purana
        brihaspati smriti
        smriti complete
        vasishtha smriti
        gayatri sahasranama
        Kamikagama
        kashyapa shilpa shastra
        apastamba dharma sutra
        baudhayana dharma sutra
        gautama dharma sutra
        vaikhanasa dharma sutra
        vasishtha dharma sutra
        vishnu dharma sutra
        gobhila grihya sutra
        jaimini grihya sutra
        kathaka grihya sutra
        shankhayana grihya sutra
        baudhayana shrauta sutra
        bharadvaja shrauta sutra
        drahyayana shrauta sutra
        Hiranyakeshi Shulba Sutra
        dhatupatha

Font: mahoregshukla
        maitrayani aranyaka
        agni purana
        complete_shiksha_book_sequence
        kanwa shiksha
        nidana shrauta sutra
        nidana sutra with accents

Font: vedic
        kashyapa_samhita
        maitrayani aranyaka
        brahma purana
        garuda purana
        markandeya purana
        Rina Vimocana nirisimha stotra
        shri natesha sahasranama
        kashyapa smriti
        vishvamitra smriti
        manava shrauta sutra


Font: mahoregsama
        nidana shrauta sutra
        nidana sutra with accents
        panchavidha sutra


Font: mahoregatharva
        harita_samhita
        kashyapa_samhita
 */