package nsf;
import java.io.BufferedWriter;
import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DefaultHandler2;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.TextFormatter;

public class ManuscriptLinks extends DefaultHandler2 {
    private String template =
"<html>\n" +
"	<head>\n" +
"		<style type='text/css'>\n" +
"			img.page {position:absolute; z-index:1;}\n" +
"			img.start {position:absolute; left:%dpx; top:%dpx; z-index:2; opacity:1.0;}\n" +
"			img.end {position:absolute; left:%dpx; top:%dpx; z-index:2; opacity:1.0;}\n" +
"		</style>\n" +
"	</head>\n" +
"	<body>\n" +
"				<img class='page' src='m/%s_crop%d.jpg'  width='%dpx' height='%dpx'/>\n" +
"				<img class='start' src='start.gif'/>\n" +
"				<img class='end' src='end.gif'/>\n" +
"	</body>\n" +
"</html>\n";

    private String dir = "C:/Users/ralph/SanskritLibrary/FridayLecture/";
    private Locator locator;
    private ArrayList<Block> blocks = new ArrayList<Block>();
    private ArrayList<Range> ranges;
    private Hashtable<String, ArrayList<Range>> h = new Hashtable<String, ArrayList<Range>>();
    public void createLinks() {
        try {
            BufferedWriter bw = IOAssistance.openWriter(dir + "MBh12ShantiParvan.roman.html");
bw.write("<html>\n");
bw.write("<head>\n");
bw.write("	<meta http-equiv='Content-Type' content='text/html; charset=utf-8'/>\n");
bw.write("	<style type='text/css'>\n");
bw.write("		a {text-decoration:none; color:black}\n");
bw.write("		a:hover {background:yellow}\n");
bw.write("	</style>\n");
bw.write("</head>\n");
bw.write("<body>\n");
bw.write("	<pre>\n");

            String text = IOAssistance.readEntireFile(dir + "MBh12ShantiParvan.roman");
            File[] files = new File(dir + "m").listFiles();
            for (File file : files) {
                if (file.getName().endsWith(".xml")) {
                    parse(file);
                    h.put(file.getName(), ranges);
                    ranges = null;
                }
            }
            files = new File(dir + "mscoll390_item490_annotations_roman").listFiles();
            Regex re = new Regex("", "(\\d+)");
            Regex refn = new Regex("", "([^/]+)$");
            int n = 0;
            for (File file : files) {
                if (file.getName().endsWith(".xml")) {
                    parse(file);
                    Collections.sort(blocks, new Block());
                    for (Block block : blocks) {
                        if (n != 0) {
                            while (n < block.start) {
                                bw.write(text.charAt(n));
                                n++;
                            }
                        } else {
                            n = block.start;
                        }
                        int pageNumber = 0;
                        re.reset(file.getName());
                        if (re.find()) {
                            pageNumber = Integer.parseInt(re.getGroup(1));
                        }
                        int cropNumber = 0;
                        ranges = h.get(file.getName());
                        Range range = null;
                        if (block.y1 < ranges.get(0).y2) {
                            range = ranges.get(0);
                            cropNumber = 1;
                        } else {
                            range = ranges.get(1);
                            cropNumber = 2;
                        }

                        int pageWidth = (range.x2 - range.x1)/4;
                        int pageHeight = (range.y2 - range.y1)/4;
                        int leftBracketX = (block.x1 - range.x1)/4 + 4;
                        int leftBracketY = (block.y1 - range.y1)/4 + 8;
                        int rightBracketX = (block.x2 - range.x1)/4 + 8;
                        int rightBracketY = (block.y2 - range.y1)/4 - 8;
                        String htmlFileName = writeHTML(cropNumber, pageNumber, pageWidth, pageHeight, leftBracketX, leftBracketY, rightBracketX, rightBracketY);
                        refn.reset(htmlFileName);
                        if (refn.find()) {
                            htmlFileName = refn.getGroup(1);
                        }
                        bw.write(String.format("<a target='m' href='%s'>", htmlFileName));
                        for (; n<=block.end; n++) {
                            bw.write(text.charAt(n));
                        }
                        bw.write("</a>");
                    }
                    blocks.clear();
                }
            }
            for (; n<text.length(); n++) {
                if (text.charAt(n) < ' ') {
                    bw.write("\n");
                    break;
                } else {
                    bw.write(text.charAt(n));
                }
            }
bw.write("</pre>\n");
bw.write("</body>\n");
bw.write("</html>\n");
            bw.close();
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
    }

    private void parse (File file) {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            // Parse the input
            SAXParser saxParser = factory.newSAXParser();
            
            saxParser.parse(file, this);
        }
        catch (Throwable t)
        {
            t.printStackTrace ();
        }
        
    }

    private String createHTMLFileName(int cropNumber, int pageNumber, int leftBracketX, int leftBracketY) {
        return String.format("%sc%d_%s_%d_%d.html",
                dir,cropNumber, TextFormatter.intToString(pageNumber, 4), leftBracketX, leftBracketY);
    }

    private String writeHTML(int cropNumber, int pageNumber, int pageWidth, int pageHeight,
                           int leftBracketX, int leftBracketY, int rightBracketX, int rightBracketY) throws Exception
    {
        String filename = createHTMLFileName(cropNumber, pageNumber, leftBracketX, leftBracketY);
        BufferedWriter bw = IOAssistance.openWriter(filename);
        bw.write(String.format(template, leftBracketX, leftBracketY, rightBracketX, rightBracketY, 
                            TextFormatter.intToString(pageNumber, 4), cropNumber, pageWidth, pageHeight));
        bw.close();
        return filename;
    }
    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw exception;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        if (qName.equals("annotation")) {
            String start = atts.getValue("start");
            String x1 = atts.getValue("x1");
            if (start != null && x1 != null) {
                String end = atts.getValue("end");
                String y1 = atts.getValue("y1");
                String x2 = atts.getValue("x2");
                String y2 = atts.getValue("y2");
                blocks.add(new Block(
                        Integer.parseInt(start),
                        Integer.parseInt(end),
                        Integer.parseInt(x1),
                        Integer.parseInt(y1),
                        Integer.parseInt(x2),
                        Integer.parseInt(y2)));
            }
        } else if (qName.equals("crops")) {
            ranges = new ArrayList<Range>();
        }
        else if (qName.equals("crop")) {
            int x1 = Integer.parseInt(atts.getValue("x1"));
            int y1 = Integer.parseInt(atts.getValue("y1"));
            int x2 = Integer.parseInt(atts.getValue("x2"));
            int y2 = Integer.parseInt(atts.getValue("y2"));
            ranges.add(new Range(x1, y1, x2, y2));
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
    }


    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
    }

    class Block implements Comparator<Block> {
        public int start;
        public int end;
        public int x1;
        public int y1;
        public int x2;
        public int y2;

        public Block () {
            
        }

        public Block(int start, int end, int x1, int y1, int x2, int y2) {
            this.start = start;
            this.end = end;
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }

        public int compare(Block o1, Block o2) {
            return o1.start - o2.start;
        }
    }

    class Range {
        public int x1;
        public int y1;
        public int x2;
        public int y2;

        public Range(int x1, int y1, int x2, int y2) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
        }
    }
}
