package nsf;

import java.io.StringReader;
import java.util.ArrayList;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;
import java.util.Hashtable;

import javax.xml.parsers.SAXParserFactory;
//import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import org.xml.sax.InputSource;

public class ParseDocument extends DefaultHandler
{
    private Locator locator;
    private Hashtable<String, TagHandler> h = new Hashtable<String, TagHandler>();
    private ArrayList<String>context = new ArrayList<String>();
    private StringBuilder sb = new StringBuilder();
    private String format;
    private SAXParser saxParser;
    private TagHandler defaultTagHandler;
    private String xml;

    public ParseDocument()
    {

        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            saxParser = factory.newSAXParser();
        }
        catch (Throwable t)
        {
            t.printStackTrace ();
        }
    }

    
    public void setDefaultTagHandler(TagHandler th)
    {
        defaultTagHandler = th;
    }

    public void parse(String xml)
    {
        this.xml = xml;
        parse(xml, null);
    }


    public void parse(String xml, Hashtable<String, TagHandler> h)
    {
        try
        {
            this.h = h;
            sb.setLength(0);

            if (xml.trim().length() == 0)           // return if nothing to do
                return;

            xml = xml.replace("&", "&amp;");

            StringReader sr = new StringReader(xml);
            InputSource is = new InputSource(sr);
            saxParser.parse(is, this);
        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
    }


    public String getHTML()
    {
        return sb.toString();
    }


    private String topContext()
    {
        int top = context.size() - 1;
        if (context.size() >= 0)
            return context.get(top);
        else
            return null;
    }

    private void pushContext(String qName)
    {
        context.add(qName);
    }

    private void popContext()
    {
        context.remove(context.size()-1);
    }

    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        pushContext(qName);
        TagHandler th = null;
        if (h != null)
            th = h.get(qName);
        if (th == null)
            th = defaultTagHandler;

        if (th != null)
        {
            sb.append(th.getStartTag(context, atts));
            format = th.getFormat(context, atts);
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        TagHandler th = null;
        if (h != null)
            th = h.get(qName);
        if (th == null)
            th = defaultTagHandler;
        
        if (th != null)
            sb.append(th.getEndTag(context));
        popContext();
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        String qName = topContext();
        TagHandler th = null;
        if (h != null)
            th = h.get(qName);
        if (th == null)
            th = defaultTagHandler;
        if (th != null && th.isPrintable())
        {
            String s = new String(ch, start, length);
            if (format != null)
                s = transcode(s);

            sb.append(th.fixup(s));
        }
    }

    private String transcode(String s)
    {
        return s;
    }

    @Override
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
