package nsf;

import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.RTF2;

import java.io.BufferedReader;
import java.io.File;
import java.io.BufferedWriter;
import java.nio.charset.Charset;
import java.util.ArrayList;

public class VedicText
{
    static public String vedicLiteratureInputDirectory = "C:/Users/ralph/Documents/PeterScharf/Vedatype/Vedic Literature";
    static public String vedicLiteratureOutputDirectory = "C:/Users/ralph/Documents/PeterScharf/Vedatype/Vedic Literature Unicode";
    public String directory;
    public String filename;
    public String type;
    public String extension;
    public String[] fonts;

    public VedicText(String directory, String filename, String type)
    {
        this(directory, filename, type, type, null);
    }

    public VedicText(String directory, String filename, String type, String[] fonts)
    {
        this(directory, filename, type, type, fonts);
    }


    public VedicText(String directory, String filename, String type, String extension, String[] fonts)
    {
        this.directory = directory;
        this.filename = filename;
        this.type = type;
        this.extension = extension;
        this.fonts = fonts;
    }

    public boolean mentionsFont(String font)
    {
        if (fonts != null)
        {
            font = font.toLowerCase();
            for (String f : fonts)
            {
                if (f.equals(font))
                    return true;
            }

            return false;
        }
        else
            return false;
    }

    public boolean validate()
    {
        try
        {
            String fn = toString();
            if (type.equals("txt"))
                return IOAssistance.fileExists(fn) && fonts == null;
            else if (type.equals("rtf"))
            {
                BufferedReader br = IOAssistance.openReader(fn);
                char[] cbuf = new char[10];
                br.read(cbuf);
                br.close();
                String s = new String(cbuf);
                return s.startsWith("{\\rtf") && fonts != null;
            }
            else
                return false;
        }
        catch (Exception ex)
        {
            return false;
        }

    }



    public ArrayList<String> getUsedFonts()
    {
        try
        {
            if (!type.equals("rtf"))
                return null;


            RTF2 rtf = new RTF2(toString(), "windows");
            return rtf.getUsedFonts();
        }
        catch (Exception ex)
        {
            return null;
        }
    }

    public String getInputFilePath()
    {
        return String.format("%s/%s/%s.%s", vedicLiteratureInputDirectory, directory, filename, extension);
    }

    public String getInputFileName()
    {
        return String.format("%s/%s/%s", vedicLiteratureInputDirectory, directory, filename);
    }

    public String getOutputFileName()
    {
        return String.format("%s/%s/%s", vedicLiteratureOutputDirectory, directory, filename);
    }

    public String getOutputFilePath()
    {
        return String.format("%s/%s/%s.%s", vedicLiteratureOutputDirectory, directory, filename, extension);
    }

    public String getInputDirectory()
    {
        return String.format("%s/%s", vedicLiteratureInputDirectory, directory);
    }

    public long getFileSize()
    {
        return new File(toString()).length();
    }

    public void copy(String ext, String charset) throws Exception
    {
        String filepath = String.format("%s.%s", getInputFileName(), ext);
        String buffer = IOAssistance.readEntireFile(filepath, charset);
        String targetfilepath = String.format("%s/%s/%s.%s", vedicLiteratureOutputDirectory, directory, filename, extension);
        BufferedWriter bw = IOAssistance.openWriter(targetfilepath);
        bw.write(buffer);
        bw.close();
    }
}
