/* Issues
 * 1. I don't have azwAdaSaSlokIgItA.*
 * 2. I don't have BhGPreface.*
 * 3. I don't have BhPSridhara.*
 * 4. I don't have catuHSlokIsaMhitA.*
 * 5. I don't have saptaSlokIgItA.*
 * 6. I don't have SivasahasranAman.*
 * 7. I don't have vizRusahasranAman.*
 * 8. I don't have VSNProse.* even though my annotations directory has a VSNProse subdirectory (however, no annotation directly refers to VSNProse.*
 * 9. Following was missing from my catalog.xml yet annotations\vizRusahasranAman\UPenn\mscoll390_item2788_wk1 directory was none empty
    <manuscript id='2788' type='Mahabharata'>
	   <directoryName>UPenn/mscoll390_item2788_wk1</directoryName>
	   <textRef>vizRusahasranAman</textRef>
	   <catalogName>UPenn Ms. Coll. 390, Item 2579</catalogName>
	   <numLeaves>0</numLeaves>
	   <numFiles>0</numFiles>
	   <title>...</title>
	   <subtitle>...</subtitle>
    </manuscript>

 */

package nsf;
import java.io.BufferedWriter;
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DefaultHandler2;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;

public class AnnotationsAnalyzer extends DefaultHandler2 {
    private String config = "C:\\Users\\ralph\\Downloads\\files2011Nov23\\config.xml";
    private String textdir = "C:\\Users\\ralph\\Downloads\\files2011Nov23\\texts\\";
    private String catalogs = "C:\\Users\\ralph\\Downloads\\files2011Nov23\\catalog\\catalog.xml";
    private String catalogsV2 = "C:\\Users\\ralph\\Downloads\\files2011Nov23\\catalog\\catalogv2.xml";
    private AnnotationsRoot annotationsRoot = null;
    private ArrayList<AnnotationInfo> annotations = new ArrayList<AnnotationInfo>();
    private Locator locator;
    private BufferedWriter out;
    int versionNumber = 0;
    String indir;
    String outdir;
    StringBuilder sb = new StringBuilder();
    SAXParserFactory factory;
    boolean getComment = false;
    AnnotationInfo currentAnnotation =  null;
    String outfile;
    String filename;
    int distance = 20;
    int lastEnd = -distance;
    int totalAdjacent = 0;
    int totalNotAdjacent = 0;
    Hashtable<String, String> type = new Hashtable<String, String>();
    String mtype = null;
    StringBuilder directoryName = new StringBuilder();
    boolean inDirectoryName = false;
    Regex directorynameRegex = new Regex("", "^([^/]+)/(.*)$");
    Regex filenameRegex = new Regex("", "^([^\\\\]+)\\\\(.*)\\\\([^\\\\]+)$");
    String textType = null;
    int startRow = 0;
    int startCol = 0;
    int endRow = 0;
    int endCol = 0;
    StringBuilder commentBuffer = new StringBuilder();
    String currentEncodingType;
    String currentTextFile;
    String currentTextFileName = "";
    ArrayList<LineInfo> currentTextIndex = new ArrayList<LineInfo>();
    Hashtable<String, ErrorReport> missing = new Hashtable<String, ErrorReport>();
    Hashtable<String, String> textMappings = new Hashtable<String, String>();

    public AnnotationsAnalyzer (String directory) throws Exception {
        this.indir = directory + "\\";
        this.outdir = directory + "v2\\";
        
        // Get mappings from config.xml (these mappings will not be required in v2)
        getTextMappings();

        // Create version 2 catalog if necessary
        if (new File(catalogsV2).exists() == false) {
            createV2Catalog();
        }

        factory = SAXParserFactory.newInstance();
        try {
            versionNumber = 0;
            SAXParser saxParser = factory.newSAXParser();
            saxParser.parse(catalogsV2, this);
        } catch (Exception ex) {

        }
    }

    private void createV2Catalog () throws Exception {
        CatalogConversion cc = new CatalogConversion(catalogs, catalogsV2, textMappings);
        cc.parse();
    }

    private void getTextMappings () {
        TextMappingParser tmp = new TextMappingParser(config);
        try {
            tmp.parse();
        } catch (Exception ex) {
            System.out.format("In getTextMappings: %s\n", ex.getMessage());
        }
        textMappings = tmp.getTextMappings();
    }

    public String convertToVersion2 () throws Exception {
        process (new File(indir), indir, outdir);
        System.out.format("distance=%d, adjacent=%d, not adjacent=%d, percent=%f",
                distance, totalAdjacent, totalNotAdjacent, (1.0*totalAdjacent)/(totalAdjacent + totalNotAdjacent));
        if (missing.size() > 0) {
            System.out.println("Problem text files (probably empty)");
            Enumeration<String> en = missing.keys();
            while (en.hasMoreElements()) {
                ErrorReport er = missing.get(en.nextElement());
                if (er.numLines > 0) {
                    System.out.format("%s, %s, %s offset:%d, numline:%d\n",
                            er.filename, er.currentTextFileName, er.type, er.offset, er.numLines);
                }
            }
        }
        return sb.toString();
    }


    public void process(File in, String inroot, String outroot) throws Exception {
        File[] files = in.listFiles();
        int count = 0;
        for (File file : files) {
            if (file.isDirectory()) {
                String encodingType = String.format("%s\\encodingType", file.getAbsolutePath());
                if (new File(encodingType).exists()) {
                    currentEncodingType = IOAssistance.readEntireFile(encodingType);
                    if (!currentEncodingType.equals("deva")) {
                        System.out.format("%s has encoding %s\n", encodingType, currentEncodingType);
                    }
                } else {
                    currentEncodingType = "deva";
                    File tfn = new File(String.format("%s%s.deva", textdir, file.getName()));
                    if (tfn.exists()) {
                        currentTextFileName = file.getName();
                        if (currentTextFileName.equals("azwAdaSaSlokIgItA")) {
                            currentTextFileName = "azwAdaSaSlokIgItA";
                        }
                        currentTextFile = IOAssistance.readEntireFile(tfn.getAbsolutePath());
                        createTextIndex(currentTextFile);
                    } 
                }
                process(file, inroot, outroot);
            } else {
                if (file.getAbsolutePath().equals("C:\\Users\\ralph\\Documents\\temp\\annotations-5-10-2011\\annotations\\MBh11StriParvan\\Brown\\37\\_0027.jpg.xml"))
                {
                    currentEncodingType = currentEncodingType + "";
                }

                if (file.getAbsolutePath().equals("C:\\Users\\ralph\\Documents\\temp\\annotations-5-10-2011\\annotations\\BhP01\\UPenn\\mscoll390_item2259_wk1\\0009.xml")) {
                    System.out.println("here");
                }
                count++;
                String fn = file.getName();
                if (fn.charAt(0) == '.' || !fn.endsWith("xml")) {
                    continue;
                }
                filename = file.getAbsolutePath();
                String oldpath = filename.substring(inroot.length());
                filenameRegex.reset(oldpath);
                if (filenameRegex.find()) {
                    textType = filenameRegex.getGroup(1);
                    String actualDirectory = filenameRegex.getGroup(2);
                    String xmlfile = filenameRegex.getGroup(3);
                    String ad = actualDirectory.replaceAll("\\\\", "/");
                    String adtype = type.get(ad);
                    if (adtype == null) {
                        adtype = "Unknown";
                    }
                    outfile = String.format("%s%s\\%s\\%s", outroot, adtype, actualDirectory, xmlfile);
                    
                    if (new File(outfile).exists()) {       // read in annotations that have already been converted to version 2
                        try {
                            if (outfile.equals("C:\\Users\\ralph\\Documents\\temp\\annotations-5-10-2011\\annotationsv2\\Mahabharata\\Brown\\37\\_0027.jpg.xml")) {
                                System.out.println("here");
                            }

                            versionNumber = 2;
                            SAXParser saxParser = factory.newSAXParser();
                            saxParser.parse(outfile, this);
                            // at this point annotationRoot and annotations have been initialized
                         } catch (Exception ex) {
                             System.out.format("Parse of %s failed with %s\n", outfile, ex.getMessage());
                         }
                    }
                } else {
                    System.out.format("%s does not match pattern\n", oldpath);
                    continue;
                }

                int fl = outfile.length() - fn.length();
                String op = outfile.substring(0, fl);
                File ofd = new File(op);
                if (!ofd.exists()) {
                    boolean b = ofd.mkdirs();
                    if (!b) {
                        System.out.format("%s(%d) mkdirs for %s failed.\n", outfile, locator.getLineNumber(), op);
                    }
                }

                try {
                    lastEnd =  -distance;
                    versionNumber = 1;
                    out = IOAssistance.openWriter(outfile);
                    SAXParser saxParser = factory.newSAXParser();
                    saxParser.parse(filename, this);
                    out.close();
                } catch (Throwable t) {
                    System.out.format("%s: %s\n", filename, t.getMessage());
                }
            }
        }
    }

    private void createTextIndex(String text) {
        LineInfo last = null;
        currentTextIndex.clear();
        int n = 0;
        int len = text.length();
        int start = -1;
        while (n < len) {
            char ch = text.charAt(n);
            if (ch != 0x0D && ch != 0x0A) {           // if it is not an EOL character
                if (start < 0) {                      // if this is start of line
                    if (last != null) {
                        last.endChar = n - 1;             // include EOL characters in last line
                    }
                    start = n;
                }
            } else if (start >= 0) {                  // at EOL character. Save line that was just ended.
                last = new LineInfo(start, n-1);
                currentTextIndex.add(last);
                start = -1;
            }
            n++;
        }
        if (start > 0) {
            currentTextIndex.add(new LineInfo(start, n-1));    // last line did not end in EOL character
        }
    }

    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw exception;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        try {
            if (versionNumber == 1) {
                if (qName.equals("annotations")) {
                    String page = atts.getValue("page");
                    if (page == null) {
                        System.out.format("%s has no page attribute but does have an dir attribute\n", filename);
                        page = atts.getValue("dir");            // why?
                    }
                    directorynameRegex.reset(page);
                    if (directorynameRegex.find()) {
                        String dirtypeValue = type.get(directorynameRegex.getGroup(2));
                        if (dirtypeValue == null) {
                            dirtypeValue="Unknown";
                        }
                        annotationsRoot = new AnnotationsRoot(dirtypeValue, atts.getValue("width"), atts.getValue("pc"));
                    } else {
                        System.out.format("%d. No type for %s\n", locator.getLineNumber(), page);
                    }
                } else if (qName.equals("annotation")) {
                    getComment = true;
                    String x1 = getAttribute(atts, "x1", "-1");
                    String y1 = getAttribute(atts, "y1", "-1");
                    String x2 = getAttribute(atts, "x2", "-1");
                    String y2 = getAttribute(atts, "y2", "-1");
                    String start = getAttribute(atts, "start", "-1");
                    String end = getAttribute(atts, "end", "-1");
                    computeRows(textType, start, end);
                    currentAnnotation = new AnnotationInfo(x1, y1, x2, y2, startRow, startCol, endRow, endCol);
                    currentAnnotation.filename = textType;
                }
            } else if (versionNumber == 2) {
                if (qName.equals("annotations")) {
                    annotationsRoot = new AnnotationsRoot(atts.getValue("dir"), atts.getValue("width"), atts.getValue("pc"));
                } else if (qName.equals("annotation")) {
                    currentAnnotation = new AnnotationInfo();
                } else if (qName.equals("manuscript")) {
                    currentAnnotation.x1 = getAttribute(atts, "x1", null);
                    currentAnnotation.y1 = getAttribute(atts, "y1", null);
                    currentAnnotation.x2 = getAttribute(atts, "x2", null);
                    currentAnnotation.y2 = getAttribute(atts, "y2", null);
                } else if (qName.equals("text")) {
                    currentAnnotation.filename = getAttribute(atts, "filename", null);
                    int idx = currentAnnotation.filename.indexOf(".");
                    if (idx != -1) {
                        currentAnnotation.filename = currentAnnotation.filename.substring(0, idx);
                    }
                    currentAnnotation.startRow = getAttribute(atts, "startRow", "-1");
                    currentAnnotation.startCol = getAttribute(atts, "startCol", "-1");
                    currentAnnotation.endRow = getAttribute(atts, "endRow", "-1");
                    currentAnnotation.endCol = getAttribute(atts, "endCol", "-1");
                } else if (qName.equals("comment")) {
                    getComment = true;
                }
            } else {                // must be processing catalog.xml
                if (qName.equals("directoryName")) {
                    inDirectoryName = true;
                }
                else if (qName.equals("manuscript")) {
                    mtype = atts.getValue("type");
                }
            }
        } catch (Exception ex) {
            throw new SAXException(String.format("%s(%d) %s", filename, locator.getLineNumber(), ex.getMessage()));
        }
    }

    private void computeRows(String textType, String start, String end) {
        if (!textType.equals(currentTextFileName)) {
            System.out.format("%s refers to text file which does not exist. Create an empty one for now.\n", filename, textType);
        }
        int _start = Integer.parseInt(start);
        int _end = Integer.parseInt(end);

        if (_start != -1) {
            if (_start < lastEnd + distance) {
                totalAdjacent++;
            } else if (lastEnd != -distance) {
                totalNotAdjacent++;
            }

            lastEnd = _end;
            if (_start != -1) {
                TextPoint tp = findTextPoint(_start);
                if (tp != null) {
                    startRow = tp.row;
                    startCol = tp.col;
                } else {                    // _start offset cannot be mapped to row and col
                    startRow = 0;
                    startCol = 0;
                    missing.put(currentTextFileName, new ErrorReport(filename, currentTextFileName, "start", _start, currentTextIndex.size()));
                }
            } else {
                startRow = -1;
                startCol = -1;
            }


            if (_end != -1) {
                TextPoint tp = findTextPoint(_end);
                if (tp != null) {
                    endRow = tp.row;
                    endCol = tp.col;
                } else {
                    endRow = 0;                     // _end offset cannot be mapped to row and col.
                    endCol = 0;
                    missing.put(currentTextFileName, new ErrorReport(filename, currentTextFileName, "end", _end, currentTextIndex.size()));
                }
            } else {
                endRow = -1;
                endCol = -1;
            }
        } else {
            startRow = -1;
            startCol = -1;
            endRow = -1;
            endCol = -1;
        }
    }

    private TextPoint findTextPoint(int offset) {
        int n = 0;
        if (currentTextIndex.size() == 0 || (currentTextIndex.size() > 0 && currentTextIndex.get(0).startChar > offset)) {
            return new TextPoint(0, 0);
        }

        for (LineInfo li : currentTextIndex) {
            if (offset >= li.startChar && offset <= li.endChar) {
                return new TextPoint(n, offset - li.startChar);
            }
            n++;
        }

        int lastRow = currentTextIndex.size() - 1;
        int col = offset - currentTextIndex.get(lastRow).startChar;
        if (col < 0) {
            col = 0;
        }
        return new TextPoint(lastRow, col);             // why is this happening?

    }

    private String getAttribute(Attributes atts, String name, String dflt) {
        String value = atts.getValue(name);
        if (value == null) {
            value = dflt;
        }

        return value;
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        try {
            if (versionNumber != 0) {
                localName = null;
            }
            if (qName.equals("annotations")) {
                if (versionNumber == 1) {
                    dumpAnnotations();          // may include annotations from other version 1 annotations
                }
            } else if (qName.equals("annotation")) {        // appropriate for either version 1 or version 2
                currentAnnotation.comment = commentBuffer.toString().trim();
                commentBuffer.setLength(0);
                annotations.add(currentAnnotation);
                getComment = false;
            } else if (qName.equals("directoryName")) {     // if parsing category.xml
                inDirectoryName = false;
                String dn = directoryName.toString();
                if (mtype != null) {
                    type.put(dn, mtype);
                    mtype = null;
                } else {
                    System.out.format("%s(%d). mtype is null\n", filename, locator.getLineNumber());
                }
                directoryName.setLength(0);
            }
        } catch (Exception ex) {
            throw new SAXException(String.format("%s(%d) %s", outfile, locator.getLineNumber(), ex.getMessage()));
        }
    }


    private void dumpAnnotations () throws Exception {
        if (annotationsRoot.dir == null) {
            System.out.format("%s has no dir attribute\n", filename);
        }
        out.write(String.format("<annotations dir='%s' width='%s' pc='%s'>\n", annotationsRoot.dir, annotationsRoot.width, annotationsRoot.pc));
        for (AnnotationInfo ai : annotations) {
            out.write(ai.toString());
        }
        out.write("</annotations>\n");
        annotations.clear();
    }


    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        if (getComment) {
            commentBuffer.append(new String(ch, start, length));
        } else if (inDirectoryName) {
            directoryName.append(new String(ch, start, length));
        }
    }

    public class AnnotationInfo {
        public String x1;
        public String y1;
        public String x2;
        public String y2;
        public String filename;
        public String startRow = "-1";
        public String startCol = "-1";
        public String endRow = "-1";
        public String endCol = "-1";
        public String comment;

        public AnnotationInfo () {

        }

        public AnnotationInfo (String x1, String y1, String x2, String y2, int startRow, int startCol, int endRow, int endCol) {
            this.x1 = x1;
            this.y1 = y1;
            this.x2 = x2;
            this.y2 = y2;
            this.startRow = Integer.toString(startRow);
            this.startCol = Integer.toString(startCol);
            this.endRow = Integer.toString(endRow);
            this.endCol = Integer.toString(endCol);
        }

        @Override
        public String toString () {
            StringBuilder sb = new StringBuilder();
            sb.append(String.format("\t<annotation>\n"));

            if (x1 != null || x2 != null) {
                sb.append(String.format("\t\t<manuscript"));
                if (x1 != null) {
                    sb.append(String.format(" x1='%s' y1='%s'", x1, y1));
                }
                if (x2 != null) {
                    sb.append(String.format(" x2='%s' y2='%s'", x2, y2));
                }
                sb.append("/>\n");
            }
            if (!startRow.equals("-1") || !endRow.equals("-1")) {
                sb.append(String.format("\t\t<text filename='%s.%s'", filename, currentEncodingType));
                if (!startRow.equals("-1")) {
                    sb.append(String.format(" startRow='%s' startCol='%s'", startRow, startCol));
                }
                if (!endRow.equals("-1")) {
                    sb.append(String.format(" endRow='%s' endCol='%s'", endRow, endCol));
                }
                sb.append("/>\n");
            }

            if (comment.length() > 0) {
                sb.append(String.format("\t\t<comment><![CDATA[%s]]></comment>\n", comment));
            }
            sb.append("\t</annotation>\n");
            
            return sb.toString();
        }
    }


    public class AnnotationsRoot {
        public String dir;
        public String width;
        public String pc;

        public AnnotationsRoot (String dir, String width, String pc) {
            this.dir = dir;
            this.width = width;
            this.pc = pc;
        }
    }

    public class LineInfo {
        public int startChar;
        public int endChar;

        public LineInfo (int startChar, int endChar) {
            this.startChar = startChar;
            this.endChar = endChar;
        }
    }

    public class TextPoint {
        public int row;
        public int col;

        public TextPoint (int row, int col) {
            this.row = row;
            this.col = col;
        }
    }

    public class ErrorReport {
        String filename;
        String currentTextFileName;
        String type;
        int offset;
        int numLines;

        public ErrorReport (String filename, String currentTextFileName, String type, int offset, int numLines) {
            this.filename = filename;
            this.currentTextFileName = currentTextFileName;
            this.type = type;
            this.offset = offset;
            this.numLines = numLines;
        }
    }
}
