package nsf;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DefaultHandler2;
import java.util.Hashtable;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.TextFormatter;

public class AttributePaths extends DefaultHandler2 {
    private ArrayList<ContextInfo>context = new ArrayList<ContextInfo>();
    private String attName;
    private Locator locator;
    private Hashtable<String, Integer> h = new Hashtable<String, Integer>();
    private SAXParser saxParser;
    private BufferedWriter outxml;
    private BufferedWriter outtext;
    private int margin=-1;
    private String blanks = "                                                    ";
    private StringBuilder sbxml = new StringBuilder("<?xml version='1.0' encoding='UTF-8'?>\n");
    private StringBuilder sbtext = new StringBuilder();
    private int currentVerse = 1;
    private int lastVerse = -1;
    private int currentSeg = 1;
    private int noADyAyaCurrentSeg = 1;
    private int currentADyAya = 0;
    private int lastADyAya = -1;
    private String currentType = "";
    private int currentTypeCount = 1;
    boolean endCommentWithLineFeed = true;      // for comments before root element
    private String textLocation = "M";

    public AttributePaths (String attName, String outxml, String outtext) throws Exception {
        this.attName = attName;
        this.outxml = IOAssistance.openWriter(outxml);
        this.outtext = IOAssistance.openWriter(outtext);
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            saxParser = factory.newSAXParser();
            saxParser.setProperty("http://xml.org/sax/properties/lexical-handler", this);
        }
        catch (Throwable t)
        {
            closeFiles();
            t.printStackTrace ();
        }
    }

    public void parse(String fileName)
    {
        InputStream is = null;
        try
        {
            is = new FileInputStream(fileName);
            saxParser.parse(is, this);
        }
        catch (Throwable t)
        {
            try {is.close();}catch (Exception ex){}
            t.printStackTrace();
        }
    }

    public Hashtable<String, Integer> getResults () {
        return h;
    }

    private void pushContext(String qName, String type)
    {
        if (type == null) {
            type = "";
        }
        ContextInfo ci = new ContextInfo(qName, type);
        context.add(ci);
    }

    private void popContext()
    {
        context.remove(context.size() - 1);
    }

    private String topType() {
        int len = context.size();
        if (len > 0) {
            return context.get(len-1).type;
        }
        else {
            return "";
        }
    }

    private String topContext() {
        int len = context.size();
        if (len > 0) {
            return context.get(len-1).qName;
        }
        else {
            return "";
        }
    }

    private void setOutputText (boolean b) {
        int len = context.size();
        if (len > 0) {
            context.get(len-1).copyText = b;
        }
    }

    private boolean getOutputText () {
        int len = context.size();
        if (len > 0) {
            return context.get(len-1).copyText;
        }
        return false;
    }

    private String buildPath () {
        StringBuilder path = new StringBuilder();
        String delim = "";
        for (ContextInfo ci : context) {
            path.append(String.format("%s%s", delim, ci.qName));
            delim = ".";
        }
        return path.toString();
    }


        // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }
    
    
    private String getMargin() {
        if (margin > 0) {
            return blanks.substring(0, margin);
        } else {
            return "";
        }
    }

    private void writeElementStart(String qName, Attributes atts) {
        margin++;
        int len = sbxml.length() - 1;
        while (len >= 0 && (sbxml.charAt(len) == ' ' || sbxml.charAt(len) == '\t')) {
            len--;
        }
        if (len >= 0) {
            sbxml.setLength(len+1);
        }

        if (len >=0 && sbxml.charAt(len) == '\n') {
            sbxml.append(getMargin());
        }
        
        sbxml.append(String.format("<%s", qName));
        if (qName.equals("l") && (currentType.equals("anuzwuB"))) {           // because l elements have no children, otherwise need to wait until seg element.
            addLineNumber();
        }

        len = atts.getLength();
        for (int i = 0; i < len; i++) {
            String name = atts.getQName(i);
            String value = atts.getValue(i);
            String type = atts.getValue("type");
            if (name.equals("n")) {
                if (type != null) {
                    if (type.equals("aDyAya")) {
                        currentADyAya = Integer.parseInt(value);
                        currentVerse = 0;
                        lastVerse = -1;
                        currentSeg = 1;
                    } else if (type.equals("verse")) {
                        value = Integer.toString(currentVerse);
                    } else if (qName.equals("lg")) {
                        currentType = type;
                        value = Integer.toString(currentSeg);
                    } else if (qName.equals("s")) {
                        value = Integer.toString(currentSeg);
                        addLineNumber();
                        currentSeg = currentSeg + 1;
                    } else if (type.equals("finalRubric") || type.equals("rubric")) {
                        value = Integer.toString(currentSeg);
                        addLineNumber();
                        currentSeg = currentSeg + 1;
                    }
                } else if (qName.equals("phr") || qName.equals("cl")) {
                        value = Integer.toString(currentSeg);
                        addLineNumber();
                        currentSeg = currentSeg + 1;
                } else if (qName.equals("speaker")) {
                    value = Integer.toString(currentSeg);
                    addLineNumber();
                    currentSeg = currentSeg + 1;
                } else if (qName.equals("seg") || qName.equals("s")) {
                    value = Integer.toString(currentSeg);
                    addLineNumber();
                    currentSeg = currentSeg + 1;
                }
                else if (qName.equals("lg")) {
                    System.out.format("Line %d: lg without type", locator.getLineNumber());
                }
            } else if ("foot".equals(type)) {
                addLineNumber();
            } else if (qName.equals("phr")) {
                    setOutputText(true);
            }
            sbxml.append(String.format(" %s='%s'", name, value));
        }
        sbxml.append(">");
    }

    private void addLineNumber () {
        sbtext.append("\n");
        sbtext.append(getLineNumber());
        sbtext.append(" ");
        setOutputText(true);
    }

    private String getLineNumber() {
        int cs = currentSeg;
        if (currentADyAya == 0) {
            cs = noADyAyaCurrentSeg;
            noADyAyaCurrentSeg++;
        }

        String ln = String.format("%s5%s%s%s%s",
                textLocation,
                TextFormatter.intToString(currentADyAya, 2),
                TextFormatter.intToString(currentADyAya==0?0:currentVerse, 2),
                TextFormatter.intToString(cs, 2),
                getCurrentSegPart());

        return ln;
    }

    private void writeText(String s) {
        if (getOutputText()) {
            int len = s.length();
            if (len > 0) {
                if (Character.isWhitespace(s.charAt(0))) {  // collapse all whitespace to a single blank
                    int start = 1;
                    while (start < len) {
                        if (Character.isWhitespace(s.charAt(start))) {
                            start++;
                        }
                        else {
                            if (start != 1) {
                                s = s.substring(start-1);
                            }
                            break;
                        }
                    }
                }

                len = sbtext.length();
                int last = len-1;
                if (Character.isWhitespace(sbtext.charAt(last))) {  // remove whitespace from end of current buffer
                    while (last >= 0) {
                        if (Character.isWhitespace(sbtext.charAt(last))) {
                            last--;
                        } else {
                            sbtext.setLength(last+2);   // include single whitespace
                            break;
                        }
                    }
                }
            }
            sbtext.append(s);
        }
    }

    private String getCurrentSegPart () {
        String sp = "\u0080";
        if (currentType.equals("anuzwuB")) {
            switch (currentTypeCount) {
                case 1: sp = "a"; break;
                case 2: sp = "c"; break;
                default: sp = "?";
            }
        } else if (!currentType.equals("")) {
            switch (currentTypeCount) {
                case 1: sp = "a"; break;
                case 2: sp = "b"; break;
                case 3: sp = "c"; break;
                case 4: sp = "d"; break;
                default: sp = "?";
            }
        }

        if (!sp.equals("\u0080")) {
            currentTypeCount++;
        }
        return sp;
    }

    private void writeElementEnd (String qName) {
        int idx = sbxml.length() - 1;
        while (idx >= 0 && (sbxml.charAt(idx) == ' ' || sbxml.charAt(idx) == '\t')) {
            idx--;
        }
        boolean slashOnly = false;
        boolean newline = true;
        if (idx >=0) {
            newline = sbxml.charAt(idx) == '\n';
            sbxml.setLength(idx+1);
        }
        if (newline) {
            sbxml.append(getMargin());
        }
        else {   // check for <foo></foo> and output </foo> instead
            if (sbxml.charAt(idx) == '>') {
                slashOnly = true;
                int len = qName.length();
                int start = idx - len - 1;
                if (sbxml.charAt(start) == '<') {
                    start++;   // move past the '<'
                    for (int i=0; slashOnly && i<len; i++) {
                        if (sbxml.charAt(start+i) != qName.charAt(i))
                            slashOnly = false;
                    }
                }
                else {
                    slashOnly = false;
                }
            } 
        }

        if (slashOnly) {
            sbxml.setLength(idx);
            sbxml.append("/>");
        } else {
            sbxml.append(String.format("</%s>", qName));
        }
        margin--;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {

    }

    private void closeFiles () {
        try {
            outxml.close();
            outtext.close();
        }
        catch (Exception ex) {

        }
    }

    @Override
    public void endDocument() throws SAXException
    {
        try 
        {
            outxml.write(sbxml.toString().replaceAll("&", "&amp;"));
            outtext.write(sbtext.toString().replace("\u0080", " "));
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        } finally {
            closeFiles();
        }
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        if (qName.equals("TEI")) {
            endCommentWithLineFeed = false;      // comments before root element need line feed, those after root element do not.
        }
        
        String type = atts.getValue("type");
        if (qName.equals("div")) {
            if (type != null) {
                if (type.equals("aMSa")) {
                    lastADyAya = currentADyAya;
                    currentADyAya = 0;
                    textLocation = "M";
                } else if (type.equals("mUla")) {
                    textLocation = "M";
                } else if (type.equals("SrIDara")) {
                    textLocation = "C";
                } else if (type.equals("verse")) {
                    currentSeg = 1;
                    if (lastVerse != -1) {
                        currentVerse = lastVerse + 1;
                    } else {
                        currentVerse = 1;
                    }
                }
                else if (type.equals("aDyAya")) {
                    currentADyAya = lastADyAya + 1;
                    currentVerse = 1;
                }
            }
        } else if (type != null) {
            if (qName.equals("seg") && type.equals("finalRubric")) {
                uri = "";
            }
        }

        pushContext(qName, atts.getValue("type"));
        writeElementStart(qName, atts);
        if (atts.getValue(attName) != null) {
            String path = buildPath();
            if (h.get(path) != null) {
            }
            else {
                h.put(path, locator.getLineNumber());
            }
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        writeElementEnd(qName);
        if (topType().equals("verse")) {
            lastVerse = currentVerse;
            currentVerse = 0;
            currentSeg = 1;
        } else if (qName.equals("lg")) {
            currentType = "";
            currentTypeCount = 1;
            currentSeg = currentSeg + 1;
        } else if (qName.equals("speaker")) {
        }
        popContext();
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        String s = new String(ch, start, length);
        sbxml.append(s);
        writeText(s);
    }

    @Override
    public void comment(char[] ch, int start, int length) throws SAXException {
        sbxml.append("<!--");
        sbxml.append(new String(ch, start, length));
        sbxml.append("-->");
        if (endCommentWithLineFeed) {
            sbxml.append("\r\n");
        }
    }

    @Override
    public void processingInstruction(String target, String data) throws SAXException
    {
        try
        {
            sbxml.append("<?");
            sbxml.append(target);
            sbxml.append(" ");
            sbxml.append(data);
            sbxml.append("?>\n");
        }
        catch (Exception ex)
        {
            throw new SAXException(ex.getMessage());
        }
    }

}
