package nsf;

import java.io.BufferedWriter;
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DefaultHandler2;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.ResourceInputStream;
import org.sanskritlibrary.XMLUtils;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.webservice.WebServices;

public class CleanupXML01 extends DefaultHandler2 //implements ErrorHandler, ContentHandler, DTDHandler, EntityResolver
{
    public String actualFile;
    StringBuilder sbxml = new StringBuilder("<?xml version='1.0' encoding='UTF-8'?>\n");
    ArrayList<String> langstack = new ArrayList<String>();
    static Hashtable<String, Integer> h = new Hashtable<String, Integer>();
    static Hashtable<String, Boolean> inEnglish = null;
    Locator locator;
    boolean endCommentWithLineFeed = true;      // for comments before root element
    SAXParser saxParser;
    String filename;
    TransformMap tm;
    static BufferedWriter debug;

    public CleanupXML01(String filename) throws Exception
    {
        this.filename = filename;
        if (inEnglish == null) {
            inEnglish = new Hashtable<String, Boolean>();
            inEnglish.put("locus", true);
            inEnglish.put("bibl", true);
            inEnglish.put("note", true);
        }
        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            saxParser = factory.newSAXParser();
            saxParser.setProperty("http://xml.org/sax/properties/lexical-handler", this);
        }
        catch (Throwable t)
        {
            System.out.format("%s: %d\n", actualFile, locator.getLineNumber());
            t.printStackTrace ();
        }
    }


    public void parse (Hashtable<String, FSM[]> fsmh, ResourceInputStream ris) throws Throwable{
        try {
            if (debug == null) {
                debug = IOAssistance.openWriter("kh_slp1.txt");
            }
            tm = new TransformMap("*:hk->slp1", ris, fsmh);
            tm.setOptionFlagged(FSM.OptionsFlagged.ASIS_DELIMS);
            debug.write(String.format("\n****%s\n", actualFile));
            saxParser.parse(new File(filename), this);
        } catch (Throwable t) {
            System.out.format("%s: %d\n", actualFile, locator.getLineNumber());
            throw t;
        }
    }

    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw exception;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        if (qName.equals("locus")) {
            qName = "locus";
        }
        endCommentWithLineFeed = false;
        if (inEnglish.get(qName) != null) {
            langstack.add("assume_eng");
        } else if (atts.getValue("xml:lang") != null) {
            langstack.add(atts.getValue("xml:lang"));
        } else if (langstack.size() > 0) {
            langstack.add(langstack.get(langstack.size()-1));
        } else {
            langstack.add("?");
        }
        sbxml.append(String.format("<%s", qName));
        int len = atts.getLength();
        for (int i = 0; i < len; i++) {
            String name = atts.getQName(i);
            String value = atts.getValue(i);
            if (name.equals("xml:lang") && value.equals("sa-Latn-x-KH")) {
                value = "sa-Latn-x-SLP1";
            }
            sbxml.append(String.format(" %s='%s'", name, value));
        }
        sbxml.append(">");

        if (langstack.get(langstack.size()-1).equals("sa-Latn-x-KH")) {
            Integer count = h.get(qName);
            if (count == null) {
                h.put(qName, new Integer(1));
            } else {
                h.put(qName, new Integer(count.intValue()+1));
            }
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        boolean slashOnly = false;
        langstack.remove(langstack.size() - 1);
        int idx = sbxml.length() - 1;
        if (sbxml.charAt(idx) == '>') {
            slashOnly = true;
            int len = qName.length();
            int start = idx - len - 1;
            if (sbxml.charAt(start) == '<') {
                start++;   // move past the '<'
                for (int i = 0; slashOnly && i < len; i++) {
                    if (sbxml.charAt(start + i) != qName.charAt(i)) {
                        slashOnly = false;
                    }
                }
            } else {
                slashOnly = false;
            }
        }

        if (slashOnly) {
            sbxml.setLength(idx);
            sbxml.append("/>");
        } else {
            sbxml.append(String.format("</%s>", qName));
        }
    }


    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        String s = new String(ch, start, length);
        String lang = langstack.get(langstack.size() - 1);
        try {
            if (s.trim().length() > 0) {
                if (lang.equals("sa-Latn-x-KH")) {
                    debug.write(String.format("%s -->\n", flatten(s)));
                    s = WebServices.transformString(s, tm);
                    debug.write(String.format("%s (%d)\n\n", flatten(s), locator.getLineNumber()));
                } else if (lang.equals("assume_eng")) {
                    debug.write(String.format("%d. assumed English: %s\n", locator.getLineNumber(), flatten(s)));
                }
            }
        } catch (Exception ex) {
            System.out.println(ex.getMessage());
        }
        sbxml.append(s);
    }

    private String flatten (String s) {
        StringBuilder sb = new StringBuilder();
        int len = s.length();
        char prevch = 0;
        for (int i=0; i<len; i++) {
            char ch = s.charAt(i);
            if (ch <= ' ') {
                if (prevch > ' ') {
                    sb.append(' ');
                }
            } else {
                sb.append(ch);
            }
            prevch = ch;
        }

        return sb.toString();
    }

    static void Cleanup (Hashtable<String, FSM[]> fsmh, ResourceInputStream ris) throws Exception {
        String dir = "C:/Users/ralph/Documents/temp/XML/";
        String targetdir = "C:/Users/ralph/Documents/PeterScharf/SanskritLibrary/siva/XML/";
        File[] files = new File(dir).listFiles();
        String blanks = "                                                                                                  ";
        int exceptionCount = 0;

        for (File file : files) {
            String name = file.getName();
            if (name.equals("penn0351.xml")) {
                continue;
            }
            if (!name.endsWith(".xml") || name.charAt(0) == '.') {
                continue;
            }
            String s = IOAssistance.readEntireFile(file.getAbsolutePath());
            String[] lines = s.split("\n");
            BufferedWriter bw = IOAssistance.openWriter("temp.xml");
            Regex re = new Regex("", "^(\\s+)(.*)$");
            for (String line : lines) {
                line = line.replaceAll("\t", " ");
                if (line.trim().length() == 0) {
                    continue;
                }
                re.reset(line);
                if (re.find()) {
                    int indentlen = re.getGroup(1).length();
                    if (indentlen % 2 != 0) {
                        exceptionCount++;
                        bw.write(line);
                        bw.write("\n");
                    } else {
                        bw.write(blanks.substring(0, re.getGroup(1).length()/2));
                        bw.write(re.getGroup(2));
                        bw.write("\n");
                    }
                }
                else {
                    bw.write(line);
                    bw.write("\n");
                }
            }
            bw.close();
            CleanupXML01 cx = new CleanupXML01("temp.xml");
            cx.actualFile = file.getAbsolutePath();
            try {
                cx.parse(fsmh, ris);
            } catch (Throwable t) {
                System.out.format("%s did not parse\n", cx.actualFile);
                t.printStackTrace();
                continue;
            }
            String targetfile = targetdir + name;
            bw = IOAssistance.openWriter(targetfile);
            String result = fixup(cx.sbxml.toString());
            try {
                new XMLUtils(XMLUtils.Types.PARSE, result);
            } catch (Throwable t) {
                System.out.format("%s not written. Has following error:\n", targetfile);
                t.printStackTrace();
                continue;
            }
            bw.write(result);
            bw.close();
        }
        CleanupXML01.debug.close();
        Enumeration<String> en = CleanupXML01.h.keys();
        while (en.hasMoreElements()) {
            System.out.println(en.nextElement());
        }

        System.out.println(exceptionCount);
    }

    static void hack () throws Exception {
        String dir = "C:/Users/ralph/Documents/PeterScharf/SanskritLibrary/siva/XML/";
        String targetdir = "C:/Users/ralph/Documents/PeterScharf/SanskritLibrary/siva/XMLhack/";
        File[] files = new File(dir).listFiles();
        String blanks = "                                                                                                  ";
        int exceptionCount = 0;

        for (File file : files) {
            String name = file.getName();
            if (!name.endsWith(".xml") || name.charAt(0) == '.') {
                continue;
            }
            String s = IOAssistance.readEntireFile(file.getAbsolutePath());
            String[] lines = s.split("\n");
            BufferedWriter bw = IOAssistance.openWriter(targetdir + name);
            Regex re = new Regex("", "^(\\s+)(.*)$");
            String lastline = "";
            int lastindent = 0;
            int linenum = 0;
            for (String line : lines) {
                linenum++;
                if (line.trim().length() == 0) {
                    continue;
                }
                re.reset(line);
                if (re.find()) {
                    int len = re.getGroup(1).length();
                    if (len > lastindent + 1) {
                        String ss = lastline.trim();
                        int ln = lastline.length() - 1;
                        if (ln > 2 && lastline.charAt(0)=='<' && lastline.charAt(ln) == '>' && lastline.charAt(ln-1) !='/') {
                            line = blanks.substring(0, lastindent + 1) + re.getGroup(2);
                        }
                    }
                    lastindent = len;
                    lastline = line.trim();
                    bw.write(line);
                    bw.write("\n");
                }
                else {
                    bw.write(line);
                    bw.write("\n");
                }
            }
            bw.close();
        }
    }

    static String fixup(String s) {
        return s.replaceAll("&", "%amp;");
    }

    @Override
    public void comment(char[] ch, int start, int length) throws SAXException {
        sbxml.append("<!--");
        sbxml.append(new String(ch, start, length));
        sbxml.append("-->");
        if (endCommentWithLineFeed) {
            sbxml.append("\r\n");
        }
    }

    @Override
    public void processingInstruction(String target, String data) throws SAXException {
        sbxml.append("<?");
        sbxml.append(target);
        sbxml.append(" ");
        sbxml.append(data);
        sbxml.append("?>\n");
    }
}
