package nsf;
import java.util.ArrayList;
import java.util.Comparator;
import org.sanskritlibrary.TextFormatter;
import org.sanskritlibrary.Regex;

public class CompareInfo implements Comparator
{
    public static int maxLength = 0;
    public static int maxOccurrences = 0;
    public String word;
    public int numOccurrences;
    public boolean gerardFound;
    public boolean jimFound;
    public ArrayList<String> files = new ArrayList<String>();
    public int sorttype;
    public String sortkeyPrefix;
    public String sortkey;
    public String _prefix;      // 3,  1,   2,     0
    public String prefix;       // GJ, G., .J, or ..
    public int lineno;      // for debugging

    public CompareInfo(int sorttype)
    {
        this.sorttype = sorttype;
    }

    public CompareInfo(String line)         // this is called in two different situations. First with line=SLP1, Second with line=SLP1 + file name abbreviations
    {
        String[] fields = line.split("\t");
        this.word = fields[0];
        if (word.length() > maxLength)
            maxLength = word.length();

        if (fields.length == 2)
        {
            fields = fields[1].split(",");
            for (int i=0; i<fields.length; i++)
            {
                files.add(fields[i]);
            }            
        }
        this.prefix = "";
    }

    public CompareInfo(String prefix, String line)
    {
        this(line);
        if (prefix.equals(".."))
            this._prefix = "0";
        else if (prefix.equals("G."))
            this._prefix = "1";
        else if (prefix.equals(".J"))
            this._prefix = "2";
        else
            this._prefix = "4";
        this.prefix = prefix;
    }


    public void addFileName(String filename)
    {
        if (!files.contains(filename))
            files.add(filename);
    }

    public void incrementWordCount(int n)
    {
        numOccurrences += n;
        if (numOccurrences > maxOccurrences)
            maxOccurrences = numOccurrences;
    }

    
    public void createSortKey()
    {
        sortkeyPrefix = _prefix +
                  TextFormatter.intToString(files.size(), 3) +
                  TextFormatter.intToString(maxLength-word.length(), 5) +
                  TextFormatter.intToString(numOccurrences, 6);
        sortkey = sortkeyPrefix + word;
    }


    public String formatPrefix()
    {
        String s = "";

        Regex re = new Regex(sortkeyPrefix, "(\\d{3})(\\d{5})(\\d{6})");
        if (re.find())
        {
           s = String.format("In %d texts, %d total occurrences, word length is %d",
                   Integer.parseInt(re.getGroup(1)),
                   numOccurrences,
                   word.length());
        }

        return s;
    }

    public int compare(Object o1, Object o2)
    {
        CompareInfo ci1 = (CompareInfo)o1;
        CompareInfo ci2 = (CompareInfo)o2;
        if (sorttype == 1)
            return ci1.word.compareToIgnoreCase(ci2.word);
        else if (sorttype == 2)
            return ci1.sortkey.compareToIgnoreCase(ci2.sortkey);
        else
            return 0;
    }

    @Override
    public String toString()
    {
      StringBuilder sb = new StringBuilder();
      sb.append(prefix);
      sb.append("   ");
      sb.append(word);
      String delim = " [";
      for (String s : files)
      {
          sb.append(delim);
          sb.append(s);
          delim = ",";
      }
      sb.append("]");
      
      return sb.toString();
    }

}
