package nsf;
import java.io.BufferedWriter;
import java.io.File;
import java.util.ArrayList;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DefaultHandler2;

import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.IOAssistance;

public class Havimsa extends DefaultHandler2 {
    private ArrayList<ElementInfo> path = new ArrayList<ElementInfo>();
    private ArrayList<SegInfo> segs = new ArrayList<SegInfo>();
    private StringBuilder speakerBuffer = new StringBuilder();
    private StringBuilder sBuffer = new StringBuilder();
    private Locator locator;
    private String currentAdyaya = "0";
    private String currentLg = "0";
    private BufferedWriter out;
    String fn;
    File file;

    public Havimsa(String fn) {
        this.fn = fn;
        file = new File(fn);

    }

    public void parse () {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            out = IOAssistance.openWriter(fn+"x");
            // Parse the input
            SAXParser saxParser = factory.newSAXParser();

            saxParser.parse(file, this);
            out.close();
        }
        catch (Throwable t)
        {
            t.printStackTrace ();
        }
    }

    private void push(String qName, Attributes atts) {
        path.add(new ElementInfo(qName, atts));
    }
    
    private ElementInfo peek () {
        if (path.size() != 0) {
            return path.get(path.size()-1);
        } else {
            return null;
        }
    }

    private String getPath () {
        StringBuilder sb = new StringBuilder();
        String delim = "";
        for (ElementInfo ei : path) {
            sb.append(String.format("%s%s", delim, ei.qName));
            delim = ".";
        }

        return sb.toString();
    }

    private ElementInfo pop() {
        ElementInfo top = null;
        if (path.size() != 0) {
            int n = path.size() - 1;
            top = path.get(n);
            path.remove(n);
        }

        return top;
    }

    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException exception) throws SAXException
    {
        throw exception;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        push(qName, atts);

        if (qName.equals("div")) {
            String type = atts.getValue("type");
            if (type != null && type.equals("aDyAya")) {
                currentAdyaya = atts.getValue("n");
            }
        } else if(qName.equals("lg")) {
            currentLg = atts.getValue("n");
        }
        else if (qName.equals("seg")) {
            segs.add(new SegInfo(locator.getLineNumber(), atts.getValue("type"), atts.getValue("n")));
        }
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        try {
            ElementInfo top = pop();
            if (qName.equals("lg")) {
                String type = top.getValue("type");
                if (type != null) {
                    if (type.equals("anuzwuB")) {
                        printSegs(true);
                        segs.clear();
                    } else {
                        printSegs(false);
                        segs.clear();
                    }
                } else {
                    printSegs(false);
                    segs.clear();
                }
            } else if (qName.equals("speaker")) {
                out.write(String.format("%s %s\n", top.getValue("n"), speakerBuffer.toString()));
                speakerBuffer.setLength(0);
            } else if (qName.equals("s")) {
                if (top.getValue("n") != null) {
                    out.write(String.format("%s %s\n", top.getValue("n"), sBuffer.toString()));
                }
                sBuffer.setLength(0);
            }
        } catch (Exception ex) {
            throw new SAXException(ex.getMessage());
        }
    }

    private void printSegs(boolean oneline) throws Exception {
        String line = "";
        if (!oneline) {
            for (SegInfo si : segs) {
                if (currentAdyaya != null && currentLg != null) {
                    line = String.format("%s.%s%s %s\n",
                            currentAdyaya, currentLg, si.n, si.sb.toString());
                } else {
                    line = String.format("%s %s\n", si.n, si.sb.toString());
                }
                out.write(line);
            }
        } else {
            if (segs.size() % 2 == 0) {
                int len = segs.size();
                for (int i = 0; i < len; i += 2) {
                    SegInfo si1 = segs.get(i);
                    SegInfo si2 = segs.get(i + 1);
                    String text1 = si1.sb.toString();
                    String text2 = si2.sb.toString();
                    String s;
                    if (text1.trim().endsWith("-")) {
                        s = String.format("%s%s", text1.substring(0, text1.length()-1), text2);
                    } else {
                        s = String.format("%s %s", text1, text2);
                    }
                    line = String.format("%s.%s%s %s\n", currentAdyaya, currentLg, si1.n, s);
                    out.write(line);
                }
            } else {
                System.out.format("oneline is true but segs.size()=%d\n", segs.size());
            }
        }
    }


    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        ElementInfo ei = peek();
        if (ei.qName.equals("seg")) {
            SegInfo si = segs.get(segs.size()-1);
            si.sb.append(new String(ch, start, length));
        } else if (ei.qName.equals("speaker")) {
            speakerBuffer.append(new String(ch, start, length));
        } else if (ei.qName.equals("s")) {
            sBuffer.append(new String(ch, start, length));
        }
    }

    class ElementInfo {
        public String qName;
        public AttributeInfo[] attInfo;
        public ElementInfo (String qName, Attributes atts) {
            this.qName = qName;
            int len = atts.getLength();
            attInfo = new AttributeInfo[len];
            for (int i=0; i<len; i++) {
                attInfo[i] = new AttributeInfo(atts.getQName(i), atts.getValue(i));
            }
        }

        public String getValue (String name) {
            for (AttributeInfo ai : attInfo) {
                if (ai.name.equals(name))
                    return ai.value;
            }

            return null;
        }
    }

    class SegInfo {
        public int linenumber;
        public String type;
        public String n;
        StringBuilder sb = new StringBuilder();

        public SegInfo (int linenumber, String type, String n) {
            this.linenumber = linenumber;
            this.type = type;
            this.n = n;
        }
    }

    class AttributeInfo {
        String name;
        String value;

        public AttributeInfo(String name, String value) {
            this.name = name;
            this.value = value;
        }
    }
}
