package nsf;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Hashtable;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.RawDecoder;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.ResourceInputStream;
import org.sanskritlibrary.TextFormatter;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.webservice.WebServices;

public class Katre {

    static void prepKatre(ResourceInputStream ris, Hashtable<String, FSM[]> fsmh) throws Exception {
        TransformMap tm = new TransformMap("*:slp1->roman", ris, fsmh);
        katreFixup02("C:/Users/ralph/Documents/astadhyayi/KATRE/KATRETRN.txt", "katresutras.txt", "", "");

        int[][] offsets = new int[8][4];
        String stroffsets = katreOffsets("katresutras.txt", offsets);
        ArrayList[] anurefs = new ArrayList[offsets[7][3] + 68];
        String anuvrtti = scharfAnuvrtti("C:/Users/ralph/Documents/PeterScharf/Panini/Panini01/anuvrtti.txt", offsets, anurefs, tm);

        katreFixup03("C:/Users/ralph/Documents/astadhyayi/KATRE/KATRE.txt", "katretranslation.txt", "", "", offsets, anurefs);
        BufferedWriter bw = IOAssistance.openWriter("katre.js");
        bw.write("P = {};\n");
        bw.write("P.history = [];\n");
        bw.write("P.currentSutra = 0;\n");
        bw.write("P.currentHistory = -1;\n");

        bw.write(stroffsets);

        String sutras = katreSutras("katresutras.txt");
        bw.write(sutras);

        String translation = katreTranslation("katretranslation.txt");
        bw.write(translation);

        bw.write(anuvrtti);

        String pratipadika = scharfPratipadika("C:/Users/ralph/Documents/PeterScharf/Panini/Panini01/pratipadika.txt",
                "C:/Users/ralph/Documents/PeterScharf/Panini/Panini01/padas.txt", offsets, tm);
        bw.write(pratipadika);

        String sutraTypes = scharfSutraTypes("C:/Users/ralph/Documents/PeterScharf/Panini/Panini01/sutras.txt", tm);
        bw.write(sutraTypes);
        
        bw.close();
    }

    static String scharfSutraTypes(String filename, TransformMap tm) throws Exception {
        StringBuilder sb = new StringBuilder();
        Hashtable<String, Integer> h = new Hashtable<String, Integer>();

        for (int pass = 0; pass < 2; pass++) {
            if (pass == 0) {
                sb.append("P.sutraTypes = [\n");
            } else {
                sb.append("P.sutraType = [\n");
            }
            BufferedReader br = IOAssistance.openReader(filename);
            String line;
            br.readLine();
            String delim = "";
            while ((line = br.readLine()) != null) {
                if (line.length() == 0) {
                    continue;
                }
                String[] fields = line.split("\t");
                if (fields.length < 7) {
                    continue;
                }
                String key = fields[6];
                if (pass == 0) {
                    Integer count = h.get(key);
                    if (count == null) {
                        count = new Integer(0);
                    }
                    h.put(key, new Integer(count.intValue() + 1));
                } else {
                    sb.append(String.format("%s%d", delim, h.get(key)));
                    delim = ",";
                }
            }

            if (pass == 0) {
                Enumeration<String> en = h.keys();
                delim = "";
                String frequency = "";
                int n = 0;
                while (en.hasMoreElements()) {
                    String key = en.nextElement();
                    sb.append(String.format("%s'%s'", delim, TextFormatter.charToEntity(WebServices.transformString(key, tm))));
                    h.get(key).toString();
                    frequency = h.get(key).toString();
                    h.put(key, n);
                    n++;
                    delim = String.format(", // %s\n", frequency);
                }
                sb.append(String.format(" // %s\n];\n", frequency));
            } else {
                sb.append("];\n");
            }
        }
        return sb.toString();
    }

    static String scharfPratipadika(String pratipadikaFile, String padasFile, int[][] offsets, TransformMap tm) throws Exception {
        StringBuilder sb = new StringBuilder();
        Sutra[] sutras = new Sutra[3983];
        for (int i = 0; i < sutras.length; i++) {
            sutras[i] = new Sutra();
        }
        BufferedReader br = IOAssistance.openReader(padasFile);
        String line;
        br.readLine();
        while ((line = br.readLine()) != null) {
            if (line.startsWith("83099")) {
                padasFile = null;
            }
            String[] fields = line.split("\t");
            int n = getSutraIdx(fields[0], offsets, "(\\d)(\\d)(\\d\\d\\d)");
            Pada p = new Pada();
            sutras[n].padas.add(p);
            p.pada = TextFormatter.charToEntity(WebServices.transformString(fields[2], tm));
            p.gender = fields[3];
            p.number = fields[4];
            p.klass = fields[5];
            p.verbID = fields[6];
            p.caseUse = TextFormatter.charToEntity(WebServices.transformString(fields[7], tm));
        }
        br.close();

        br = IOAssistance.openReader(pratipadikaFile);
        br.readLine();
        while ((line = br.readLine()) != null) {
            String[] fields = line.split("\t");
            int n = getSutraIdx(fields[0], offsets, "(\\d)(\\d)(\\d\\d\\d)");
            Sutra sutra = sutras[n];
            int pn = Integer.parseInt(fields[1]);
            String pratn = fields[2];
            if (!(pn - 1 < sutra.padas.size())) {
                System.out.println(line);
                continue;
            }
            Pada pada = sutra.padas.get(pn - 1);
            PadaPart pp = new PadaPart();
            pada.padaParts.add(pp);
            String indent = "";
            if (pratn.equals("0")) {
                pp.level = 0;
            } else if (pratn.length() == 1) {
                indent = "..";
                pp.level = 1;
            } else {
                pp.level = 1 + (pratn.length() - 1) / 2;
                indent = ".........................................................".substring(0, pp.level * 2);
            }
            pp.pratipradika = indent + TextFormatter.charToEntity(WebServices.transformString(fields[3], tm));
            pp.compoundType = TextFormatter.charToEntity(WebServices.transformString(fields[4], tm));
            pp.lexID = TextFormatter.charToEntity(WebServices.transformString(fields[5], tm));
            pp.sublexID = TextFormatter.charToEntity(WebServices.transformString(fields[6], tm));
            pp.marker = TextFormatter.charToEntity(WebServices.transformString(fields[7], tm));
            pp.pronounciation = fields[8];
            pp.tapara = fields[9].substring(0, 1);
        }

        sb.append("\n\nP.pada=[\n");
        String delim = "";
        for (int i = 0; i < sutras.length; i++) {
            sb.append(delim);
            delim = ",\n";
            sb.append(sutras[i].toString());
        }
        sb.append("\n];\n");

        br.close();

        return sb.toString();
    }

    static String scharfAnuvrtti(String filename, int[][] offsets, ArrayList[] anurefs, TransformMap tm) throws Exception {
        StringBuilder sb = new StringBuilder();
        BufferedReader br = IOAssistance.openReader(filename);
        String line = null;
        int currentIdx = -1;
        br.readLine();   // skip headers
        String delim = "";
        sb.append("P.an = [\n");
        int lastidx = -1;
        boolean closeArray = false;
        while ((line = br.readLine()) != null) {
            String[] fields = line.split("\t");
            int idx = getSutraIdx(fields[0], offsets, "(\\d)(\\d)(\\d+)");
            if (idx != lastidx && idx != lastidx + 1) {
                if (closeArray) {
                    sb.append("],\n");
                    closeArray = false;
                }
                while (lastidx+1 != idx) {
                    lastidx++;
                    sb.append("[],\n");
                }
            }
            lastidx = idx;
            if (idx == -1) {
                System.out.format("Scharf anuvrtti: Illegal sutra number: %s\n", line);
                continue;
            }
            String pada = fields[2];
            pada = WebServices.transformString(pada, tm);
            pada = TextFormatter.charToEntity(pada);
            int refidx = getSutraIdx(fields[3], offsets, "(\\d)(\\d)(\\d+)");
            if (refidx == -1) {
                System.out.format("Scharf anuvritti: Illegal anuvrtti sutra number: %s\n", line);
                continue;
            }
            if (idx != currentIdx) {
                if (currentIdx != -1 && closeArray) {

                    closeArray = false;
                    sb.append("],\n");
                }
                delim = "";
                sb.append(String.format("["));
                closeArray = true;
                currentIdx = idx;
            }
            if (anurefs[idx] == null) {
                anurefs[idx] = new ArrayList();
            }
            anurefs[idx].add(refidx);
            sb.append(String.format("%s{p:'%s', n:%d}", delim, pada, refidx));
            delim = ",";
        }
        if (closeArray) {
            sb.append("]\n");
        }
        sb.append("];\n");

        return sb.toString();
    }

    static int getSutraIdx(String sn, int[][] offsets, String pattern) {
        Regex re = new Regex(sn, pattern);
        boolean okay = true;
        if (re.find()) {
            int achapter = Integer.parseInt(re.getGroup(1));
            int apada = Integer.parseInt(re.getGroup(2));
            if (achapter == 0 || achapter > 8 || apada == 0 || apada > 4) {
                return -1;
            }
            int nextChapter = 0;
            int nextPada = 0;
            int asutraNumber = Integer.parseInt(re.getGroup(3));
            if (achapter != 8 && apada != 4) {
                if (apada == 4) {
                    nextChapter = achapter + 1;
                    nextPada = 1;
                } else {
                    nextChapter = achapter;
                    nextPada = apada + 1;
                }

                okay = asutraNumber <= offsets[nextChapter - 1][nextPada - 1] - offsets[achapter - 1][apada - 1];
            } else if (achapter == 8 && apada == 4 && asutraNumber > 68) {
                okay = false;
            }

            if (okay) {
                return offsets[achapter - 1][apada - 1] + asutraNumber - 1;
            } else {
                return -1;
            }
        } else {
            return -1;
        }
    }

    static String katreTranslation(String filename) throws Exception {
        StringBuilder sb = new StringBuilder();
        BufferedReader br = IOAssistance.openReader(filename);
        String line = null;
        int n = 0;
        sb.append("P.tr = [\n");
        while ((line = br.readLine()) != null) {
            if (line.length() > 0) {
                line = line.replace("'", "\\'");
                int idx = line.indexOf(" ");
                line = line.substring(idx + 1);
                sb.append(String.format("'%s',\n", line.trim()));
                n++;
            }
        }
        sb.append("];\n");
        br.close();
        return sb.toString();
    }

    static String katreSutras(String filename) throws Exception {
        StringBuilder sb = new StringBuilder();
        BufferedReader br = IOAssistance.openReader(filename);
        String line = null;
        int n = 0;
        sb.append("P.su = [\n");
        while ((line = br.readLine()) != null) {
            String delim = "'";
            if (line.length() > 0) {
                if (line.indexOf("'") != -1) {
                    delim = "\"";
                }
                sb.append(String.format("%s%s%s,\n", delim, line.trim(), delim));
                n++;
            }
        }
        sb.append("];\n");
        br.close();
        return sb.toString();
    }

    static String katreOffsets(String filename, int[][] offsets) throws Exception {
        StringBuilder sb = new StringBuilder();
        BufferedReader br = IOAssistance.openReader(filename);
        String line = null;
        String currentChapter = "1";
        String currentPada = "1";
        String lastSutra = "";
        int sutraCount = 0;
        int cumulativeCount = 0;
        sb.append("P.offset = [];\n");
        sb.append("P.offset[1] = [];\n");
        sb.append("P.offset[2] = [];\n");
        sb.append("P.offset[3] = [];\n");
        sb.append("P.offset[4] = [];\n");
        sb.append("P.offset[5] = [];\n");
        sb.append("P.offset[6] = [];\n");
        sb.append("P.offset[7] = [];\n");
        sb.append("P.offset[8] = [];\n");
        Regex re = new Regex("", "(\\d)\\.(\\d)\\.([^\\s]+)");
        while ((line = br.readLine()) != null) {
            re.reset(line);
            if (re.find()) {
                String chapter = re.getGroup(1);
                String pada = re.getGroup(2);
                if (chapter.equals(currentChapter) && pada.equals(currentPada)) {
                    lastSutra = re.getGroup(3);
                    sutraCount++;
                } else {
                    int ls = Integer.parseInt(lastSutra);
                    if (ls != sutraCount) {
                        System.out.format("%s.%s observed:%d, last:%d;\n", currentChapter, currentPada, sutraCount, ls);
                    }

                    sb.append(String.format("P.offset[%s][%s] = %d;  // %d sutras\n", currentChapter, currentPada, cumulativeCount, sutraCount));
                    offsets[Integer.parseInt(currentChapter) - 1][Integer.parseInt(currentPada) - 1] = cumulativeCount;
                    currentChapter = chapter;
                    currentPada = pada;
                    cumulativeCount += sutraCount;
                    sutraCount = 1;
                }
            }
        }

        sb.append(String.format("P.offset[%s][%s] = %d;  // %d sutras\n", currentChapter, currentPada, cumulativeCount, sutraCount));
        offsets[Integer.parseInt(currentChapter) - 1][Integer.parseInt(currentPada) - 1] = cumulativeCount;

        return sb.toString();
    }

    static void katreFixup01() throws Exception {

//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/ASTA.txt";
//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRETRN.txt";
        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRE.txt";
        BufferedReader br = null;
        BufferedInputStream bis = null;
        InputStreamReader isr = null;
        FileInputStream fis = new FileInputStream(filename);
        bis = new BufferedInputStream(fis);
        isr = new InputStreamReader(bis, new RawDecoder(Charset.forName("UTF-8"), 1F, 1F));
        br = new BufferedReader(isr);
        String[] map = getKatreMap();
        Hashtable<Character, String> h = new Hashtable<Character, String>();
        Hashtable<Character, Integer> hcount = new Hashtable<Character, Integer>();
        String line = null;
        int lineno = 0;
        while ((line = br.readLine()) != null) {
            lineno++;
            int len = line.length();
            for (int i = 0; i < len; i++) {
                char ch = line.charAt(i);
                if (map[ch] != null) {
                    continue;
                }
                if (h.get(ch) == null) {
                    int idx = line.indexOf(' ');
                    //h.put(ch, line.substring(0, idx));
                    h.put(ch, Integer.toString(lineno));
                }
                Integer cnt = hcount.get(ch);
                if (cnt == null) {
                    cnt = new Integer(0);
                }
                hcount.put(ch, new Integer(cnt.intValue() + 1));
            }
        }
        br.close();
        Enumeration<Character> en = h.keys();
        ArrayList<Character> a = new ArrayList<Character>();
        while (en.hasMoreElements()) {
            a.add(en.nextElement());
        }

        Collections.sort(a);

        for (Character ch : a) {
            System.out.format("<tr><td>***0x%s</td><td>%s</td><td>%d</td><td>&nbsp;</td><td>&nbsp;</td></tr>\n",
                    TextFormatter.toHexString((int) ch), h.get(ch), hcount.get(ch).intValue());
        }
    }

    static void katreFixup02(String in, String out, String startDelim, String endDelim) throws Exception {
        String[] map = getKatreMap();
        StringBuilder sb = new StringBuilder();
//       String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/ASTA.txt";   // entire book
//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRETRN.txt";  // transliteration of sutras
//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRE.txt";   // translation of sutras
        BufferedReader br = null;
        BufferedInputStream bis = null;
        InputStreamReader isr = null;
        FileInputStream fis = new FileInputStream(in);
        bis = new BufferedInputStream(fis);
        isr = new InputStreamReader(bis, new RawDecoder(Charset.forName("UTF-8"), 1F, 1F));
        br = new BufferedReader(isr);

        Hashtable<Character, String> h = new Hashtable<Character, String>();
        Hashtable<Character, Integer> hcount = new Hashtable<Character, Integer>();
        String line = null;
        while ((line = br.readLine()) != null) {
            if (line.startsWith("7.2.19")) {
                bis = null;
            }
            sb.append(startDelim);
            int len = line.length();
            for (int i = 0; i < len; i++) {
                char ch = line.charAt(i);
                if (ch == 0x92) {
                    ch = 0x92;
                }
                if (map[ch] != null) {
                    sb.append(map[ch]);
                } else if (ch > 0x7f) {
                    sb.append(String.format("<0x%s>", TextFormatter.toHexString(ch)));
                } else {
                    sb.append(ch);
                }
            }
            sb.append(String.format("%s\n", endDelim));
        }
        br.close();
        BufferedWriter bw = IOAssistance.openWriter(out);
        bw.write(sb.toString());
        bw.close();
    }

    static void katreFixup03(String in, String out, String startDelim, String endDelim, int[][] offsets, ArrayList[] anurefs) throws Exception {
        LinkMap sorter = new LinkMap();
        ArrayList<LinkMap> linkmap = new ArrayList<LinkMap>();
        String[] map = getKatreMap();
        StringBuilder sb = new StringBuilder();
        StringBuilder sblink = new StringBuilder();
//       String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/ASTA.txt";   // entire book
//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRETRN.txt";  // transliteration of sutras
//        String filename = "C:/Users/ralph/Documents/astadhyayi/KATRE/KATRE.txt";   // translation of sutras
        BufferedReader br = null;
        BufferedInputStream bis = null;
        InputStreamReader isr = null;
        FileInputStream fis = new FileInputStream(in);
        bis = new BufferedInputStream(fis);
        isr = new InputStreamReader(bis, new RawDecoder(Charset.forName("UTF-8"), 1F, 1F));
        br = new BufferedReader(isr);
        Regex renum = new Regex("", "(^[\\d.]+)");
        Regex redhatu = new Regex("", "([IVX]+\\s+[\\d-]+)");
        Regex reref = new Regex("", "(\\d+(?:\\.\\d+(?:\\.\\d+)?)?)");
        Regex rerange = new Regex("", "(\\d+(?:\\.\\d+(?:\\.\\d+)?)?)\\s*-\\s*(\\d+(?:\\.\\d+(?:\\.\\d+)?)?)");
        String sn = null;
        int snoffset = 0;
        Hashtable<Character, String> h = new Hashtable<Character, String>();
        String line = null;
        ArrayList<DhatuLocation> dp = new ArrayList<DhatuLocation>();
        while ((line = br.readLine()) != null) {
            linkmap.clear();
            sblink.setLength(0);
            if (line.startsWith("1.1.11")) {
                line = line + "";
            }
            dp.clear();
            renum.reset(line);
            if (renum.find()) {
                sn = renum.getGroup(1);
                snoffset = getSutraIdx(sn, offsets, "(\\d)\\.(\\d)\\.(\\d+)");
                if (snoffset == -1) {
                    System.out.format("Katre translation: Illegal sutra number: %s\n", line);
                    continue;
                }
            }

            // Remember where references to DhatuPata are
            redhatu.reset(line);
            while (redhatu.find()) {
                int start = redhatu.getOffset(1);
                int end = start + redhatu.getGroup(1).length();
                dp.add(new DhatuLocation(start, end));
            }

            // expand ranges (e.g., 1.2.8-10), some such as 1.2.30-3 were editted by hand to, e.g., 1.2.30-33
            rerange.reset(line);
            while (rerange.find()) {
                int rangestart = rerange.getOffset(1);

                RefInfo ri01 = getSutraReference(line, sn, rerange, 1, offsets, anurefs[snoffset]);

                String ref02 = null;
                if (ri01.ref != null) {
                    ref02 = ri01.ref;
                } else {
                    ref02 = sn;
                }
                RefInfo ri02 = getSutraReference(line, ref02, rerange, 2, offsets, anurefs[snoffset]);
                if (ri01.n > ri02.n) {
                    System.out.print("--->");
                }
                String link = String.format("<a href='#' onclick=lc(%d)>%s</a>", ri01.n, ri01.original);
                linkmap.add(new LinkMap(link, rerange.getOffset(1), rerange.getOffset(1) + ri01.original.length()));
                link = String.format("<a href='#' onclick=lc(%d)>%s</a>", ri02.n, ri02.original);
                linkmap.add(new LinkMap(link, rerange.getOffset(2), rerange.getOffset(2) + ri02.original.length()));
                System.out.format("***%s-%s: %s\n",
                        ri01.ref == null ? "?" : ri01.ref,
                        ri02.ref == null ? "?" : ri02.ref,
                        line);
            }

            // Expand normal references
            reref.reset(line);
            boolean bFirst = true;
            while (reref.find()) {
                int start = reref.getOffset(1);

                if (!bFirst) {
                    boolean ignore = false;
                    for (DhatuLocation d : dp) {
                        if (start >= d.start && start < d.end) {
                            ignore = true;
                            break;
                        }
                    }
                    if (!ignore) {
                        RefInfo ri = getSutraReference(line, sn, reref, 1, offsets, anurefs[snoffset]);
                        if (!ri.makeLink) {
                            continue;
                        } else {
                            String link = String.format("<a href='#' onclick=lc(%d)>%s</a>", ri.n, ri.original);
                            int ofs = reref.getOffset(1);
                            boolean found = false;
                            for (LinkMap lm : linkmap) {
                                if (lm.start == ofs) {
                                    found = true;
                                    break;
                                }
                            }
                            if (!found) {
                                linkmap.add(new LinkMap(link, reref.getOffset(1), reref.getOffset(1) + ri.original.length()));
                            }

                            System.out.format("%s: %s\n", sn, ri.ref);
                        }
                    }
                } else {
                    bFirst = false;
                }
            }

            Collections.sort(linkmap, sorter);
            insertLinks(line, linkmap, sblink);

            sb.append(startDelim);
            int len = sblink.length();
            for (int i = 0; i < len; i++) {
                char ch = sblink.charAt(i);
                if (map[ch] != null) {
                    sb.append(map[ch]);
                } else if (ch > 0x7f) {
                    sb.append(String.format("<0x%s>", TextFormatter.toHexString(ch)));
                } else {
                    sb.append(ch);
                }
            }
            sb.append(String.format("%s\n", endDelim));
        }
        br.close();
        BufferedWriter bw = IOAssistance.openWriter(out);
        bw.write(sb.toString());
        bw.close();
    }

    static void insertLinks(String line, ArrayList<LinkMap> linkmap, StringBuilder sb) {
        int linept = 0;
        for (LinkMap lm : linkmap) {
            if (linept < lm.start) {
                sb.append(line.substring(linept, lm.start));
                linept = lm.end;
            }
            sb.append(lm.link);
        }
        if (linept < line.length()) {
            sb.append(line.substring(linept));
        }
    }

    static RefInfo getSutraReference(String line, String sn, Regex reref, int gn, int[][] offsets, ArrayList a) {
        String original = reref.getGroup(gn);
        String ref = null;
        int ofs = reref.getOffset(gn);
        char ch = line.charAt(ofs - 1);
        if (ch == '`' || ch == '\'') {
            return new RefInfo(original, null, -1, false);
        }
        int levels = 0;
        int len = original.length();
        for (int i = 0; i < len; i++) {
            if (original.charAt(i) == '.') {
                levels++;
            }
        }

        if (levels == 0) {
            String[] fields = sn.split("\\.");
            ref = String.format("%s.%s.%s", fields[0], fields[1], original);
        } else if (levels == 1) {
            String[] fields = sn.split("\\.");
            ref = String.format("%s.%s", fields[0], original);
        } else {
            ref = original;
        }


        int n = getSutraIdx(ref, offsets, "(\\d)\\.(\\d)\\.(\\d+)");          // sutra reference in Katre's translation
        if (n != -1) {
            // Now see if this sutra reference is also in Peter's anuvrtti. If it is then do not make it an anchor.
            if (a != null) {
                len = a.size();
                for (int i = 0; i < len; i++) {
                    if (((Integer) a.get(i)) == n) {     // Peter already mentions this
                        return new RefInfo(original, ref, n, false);
                    }
                }
            }
        } else {
            System.out.format("Katre translation: Illegal sutra reference(%s) number: %s\n", ref, line);
            return new RefInfo(original, null, -1, false);
        }

        return new RefInfo(original, ref, n, true);
    }

    static String[] getKatreMap() {
        String[] katremap = new String[256];
        katremap[0x80] = "&#x2070;";
        katremap[0x81] = "&#x00B9;";
        katremap[0x82] = "&#x00B2;";
        katremap[0x83] = "&#x00B3;";
        katremap[0x84] = "&#x2074;";
        katremap[0x85] = "&#x2075;";
        katremap[0x86] = "&#x2076;";
        katremap[0x87] = "&#x2077;";
        katremap[0x88] = "&#x2078;";
        katremap[0x89] = "&#x2079;";
        katremap[0x8a] = "{^v}";
        katremap[0x8b] = "{^y}";
        katremap[0x8c] = "&#x0304;&#x0301;";
        katremap[0x8d] = "&#x0301;";
        katremap[0x8e] = "&#x2192;";
        katremap[0x8f] = "&#x00B0;";
        katremap[0x90] = "0&#x0338;";
        katremap[0x91] = "&#x00B1;";
        katremap[0x92] = "A&#x0301;";
        katremap[0x93] = "A&#x0304;";
        katremap[0x94] = "A&#x0300;";
        katremap[0x95] = "A&#x0304;";
        katremap[0x96] = "A&#x0303;";
        katremap[0x97] = "D&#x0323;";
        katremap[0x98] = "E&#x0301;";
        katremap[0x99] = "E&#x0300;";
        katremap[0x9a] = "I&#x0301;";
        katremap[0x9b] = "I&#x0304;&#x0301;";
        katremap[0x9c] = "I&#x0300;";
        katremap[0x9d] = "I&#x0304;&#x0300;";
        katremap[0x9e] = "I&#x0304;";
        katremap[0x9f] = "L&#x0301;&#x0325;";
        katremap[0xa1] = "L&#x0300;&#x0325;";
        katremap[0xa2] = "L&#x0325;";
        katremap[0xa3] = "N&#x0307;";
        katremap[0xa4] = "N&#x0323;";
        katremap[0xa5] = "N&#x0303;";
        katremap[0xa6] = "N&#x0303;";
        katremap[0xa7] = "O&#x0301;";
        katremap[0xa8] = "R&#x0301;&#x0325;";
        katremap[0xa9] = "R&#x0300;&#x0325;";
        katremap[0xaa] = "R&#x0325;";
        katremap[0xab] = "S&#x0301;";
        katremap[0xac] = "S&#x0323;";
        katremap[0xad] = "T&#x0323;";
        katremap[0xae] = "T";
        katremap[0xaf] = "U&#x0301;";
        katremap[0xb0] = "U&#x0304;&#x0300;";
        katremap[0xb1] = "U&#x0331;";
        katremap[0xb2] = "U&#x0300;";
        katremap[0xb3] = "U&#x0304;&#x0300;";
        katremap[0xb4] = "U&#x0304;";
        katremap[0xb5] = "U&#x0303;";
        katremap[0xb6] = "a&#x0301;";
        katremap[0xb7] = "a&#x0304;";
        katremap[0xb8] = "a&#x0301;&#x0331;";
        katremap[0xb9] = "a&#x0304;";
        katremap[0xba] = "a&#x0331;";
        katremap[0xbb] = "a&#x0323;";
        katremap[0xbc] = "a&#x0300;";
        katremap[0xbd] = "a&#x0304;";
        katremap[0xbf] = "a&#x0304;&#x0331;";
        katremap[0xbe] = "a&#x0304;";
        katremap[0xc0] = "a&#x0303;";
        katremap[0xc1] = "a&#x0304;&#x0303;";
        katremap[0xc2] = "d&#x0331;";
        katremap[0xc3] = "d&#x0323;";
        katremap[0xc4] = "e&#x0301;";
        katremap[0xc5] = "e&#x0331;";
        katremap[0xc6] = "e&#x0323;";
        katremap[0xc7] = "e&#x0300;";
        katremap[0xc8] = "e&#x0304;";
        katremap[0xc9] = "h&#x0323;";
        katremap[0xca] = "i&#x0301;";
        katremap[0xcb] = "i&#x0331;";
        katremap[0xcc] = "i&#x0304;";
        katremap[0xcd] = "i&#x0331;";
        katremap[0xce] = "i&#x0323;";
        katremap[0xcf] = "i&#x0300;";
        katremap[0xd0] = "&#x0304;&#x0300;";
        katremap[0xd1] = "i&#x0304;";
        katremap[0xd2] = "i&#x0304;&#x0331;";
        katremap[0xd3] = "i&#x0303;";
        katremap[0xd4] = "i&#x0304;&#x0303;";
        katremap[0xd5] = "i&#x0304;";
        katremap[0xd6] = "l&#x0301;";
        katremap[0xd7] = "l&#x0323;";
        katremap[0xd8] = "l&#x0304;&#x0325";
        katremap[0xd9] = "l&#x0325;";
        katremap[0xda] = "l&#x0303;";
        katremap[0xdb] = "m&#x0307;";
        katremap[0xdc] = "n&#x0301;";
        katremap[0xdd] = "n&#x0307;";
        katremap[0xde] = "n&#x0323;";
        katremap[0xdf] = "n&#x0303;";
        katremap[0xe0] = "o&#x0301;";
        katremap[0xe1] = "o&#x0331;";
        katremap[0xe2] = "o&#x0300;";
        katremap[0xe3] = "r&#x0304;";
        katremap[0xe4] = "r&#x0301;";
        katremap[0xe5] = "r&#x0304;";
        katremap[0xe6] = "r&#x0325;";
        katremap[0xe7] = "s&#x0301;";
        katremap[0xe8] = "s&#x0323;";
        katremap[0xe9] = "t&#x0331;";
        katremap[0xea] = "t&#x0323;";
        katremap[0xeb] = "u&#x0301;";
        katremap[0xec] = "u&#x0331;&#x0301;";
        katremap[0xed] = "u&#x0304;";
        katremap[0xee] = "u&#x0331;";
        katremap[0xef] = "u&#x0300;";
        katremap[0xf0] = "u&#x0304;&#x0300;";
        katremap[0xf1] = "u&#x0304;";
        katremap[0xf2] = "u&#x0304;&#x0331;";
        katremap[0xf3] = "u&#x0303;";
        katremap[0xf4] = "u&#x0304;&#x0303;";
        katremap[0xf5] = "v&#x0303;";
        katremap[0xf6] = "y&#x0301;";
        katremap[0xf7] = "y&#x0331;";
        katremap[0xf8] = "y&#x0300;";
        katremap[0xf9] = "y&#x0304;";
        katremap[0xfa] = "y&#x0303;";

        return katremap;
    }

    static class DhatuLocation {

        int start;
        int end;

        public DhatuLocation(int start, int end) {
            this.start = start;
            this.end = end;
        }
    }

    static class RefInfo {

        String original;
        String ref;
        int n;
        boolean makeLink;

        public RefInfo(String original, String ref, int n, boolean makeLink) {
            this.original = original;
            this.ref = ref;
            this.n = n;
            this.makeLink = makeLink;
        }
    }

    static class LinkMap implements Comparator<LinkMap> {

        String link;
        int start;
        int end;

        public LinkMap() {
        }

        public LinkMap(String link, int start, int end) {
            this.link = link;
            this.start = start;
            this.end = end;
        }

        public int compare(LinkMap o1, LinkMap o2) {
            return o1.start - o2.start;
        }
    }

    static class Sutra {

        ArrayList<Pada> padas = new ArrayList<Pada>();

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();

            String delim = "";
            sb.append("[");
            for (Pada pada : padas) {
                sb.append(delim);
                delim = ",";
                sb.append(pada.toString());
            }
            sb.append("]");
            return sb.toString();
        }
    }

    static class Pada {

        String pada;
        String gender;
        String number;
        String klass;
        String verbID;
        String caseUse;
        ArrayList<PadaPart> padaParts = new ArrayList<PadaPart>();

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            String delim = "'";
            if (pada.contains("'")) {
                delim = "\"";
            }
            sb.append(String.format("{p:%s%s%s", delim, pada, delim));
            if (gender.length() > 0) {
                sb.append(String.format(",g:'%s'", gender));
            }
            if (number.length() > 0) {
                sb.append(String.format(",n:'%s'", number));
            }
            if (klass.length() > 0) {
                sb.append(String.format(",c:'%s'", klass));
            }
            if (verbID.length() > 0) {
                sb.append(String.format(",v:'%s'", verbID));
            }
            if (caseUse.length() > 0) {
                sb.append(String.format(",u:'%s'", caseUse));
            }

            sb.append(",pps: [");
            delim = "";
            for (PadaPart padaPart : padaParts) {
                sb.append(delim);
                delim = ",";
                sb.append(padaPart.toString());
            }
            sb.append("]}");
            return sb.toString();
        }
    }

    static class PadaPart {

        int level;
        String pratipradika;
        String compoundType;
        String lexID;
        String sublexID;
        String marker;
        String pronounciation;
        String tapara;

        @Override
        public String toString() {
            StringBuilder sb = new StringBuilder();
            sb.append("{");
            String delim = "'";
            if (pratipradika.contains("'")) {
                delim = "\"";
            }
            sb.append(String.format("pp:%s%s%s", delim, pratipradika, delim));
            if (compoundType.length() > 0) {
                sb.append(String.format(",c:'%s'", compoundType));
            }
            if (lexID.length() > 0) {
                sb.append(String.format(",l:'%s'", lexID));
            }
            if (sublexID.length() > 0) {
                sb.append(String.format(",s:'%s'", sublexID));
            }
            if (marker.length() > 0) {
                sb.append(String.format(",m:'%s'", marker));
            }
            if (pronounciation.length() > 0) {
                sb.append(String.format(",p:'%s'", pronounciation));
            }
            if (tapara.length() > 0) {
                sb.append(String.format(",t:'%s'", tapara));
            }
            sb.append("}");
            return sb.toString();
        }
    }
}
