package nsf;

import java.util.Hashtable;
import org.xml.sax.Attributes;

public class PathInfo
{
    boolean ofInterest = false;
    StringBuilder currentKey = new StringBuilder();
    StringBuilder currentPath = new StringBuilder();
    StringBuilder currentBuffer = null;
    String containerPath = null;
    Hashtable<String, ElementInfo> paths = new Hashtable<String, ElementInfo>();
    Hashtable<String, String> keys = new Hashtable<String, String>();
    Hashtable<String, Boolean> containers = new Hashtable<String, Boolean>();
    Hashtable<String, ElementHandler> handlers = new Hashtable<String, ElementHandler>();
    ElementHandler defaultHandler;
    int debug = 0;

    public void put(String path)
    {
        paths.put(path, new ElementInfo());
    }

    public void key(String path, String att)
    {
        keys.put(path, att);
    }

    public void container(String path)
    {
        if (paths.get(path) == null)
            paths.put(path, new ElementInfo());
        containers.put(path, true);
    }

    public void handler(String path, ElementHandler eh)
    {
        if (paths.get(path) == null)
            paths.put(path, new ElementInfo());
        handlers.put(path, eh);
    }

    public void setDefaultHandler(ElementHandler eh)
    {
        this.defaultHandler = eh;
    }

    public void pushElement(String qName, Attributes atts)
    {
        String delim = "";
        if (currentKey.length() > 0)
            delim = ".";

        currentKey.append(String.format("%s%s", delim, qName));         // used to look up values in keys, containers and handlers hash tables.
        if (currentKey.toString().equals("TEI.text.body.div"))
            debug = debug++ - 1;
        String key = "";
        String attkey = keys.get(currentKey.toString());
        if (attkey != null)                     // need to append the value of the key attribute to the path
        {
            String value = atts.getValue(attkey);
            if (value != null)
                key = String.format("[%s='%s']", attkey, value);
            else
                key = String.format("[%s='*']", attkey);        // key attribute is missing, use '*'
        }
        else
            key = "";

        currentPath.append(String.format("%s%s%s", delim, qName, key));

        if (containerPath != null)
            copyTag(qName, atts);
        else
        {
            if (containers.get(currentKey.toString()) != null)              // this is a container
            {
                containerPath = currentPath.toString();
                currentBuffer = paths.get(currentPath.toString()).buffer;   // this buffer will be used for all children
                copyTag(qName, atts);
            }
            else
            {
                ofInterest = isOfInterest();
                if (ofInterest)
                {
                    if (containerPath != null)
                    {
                        System.out.println(String.format("Warning:  In container %s for item of interest %s\n",
                                            containerPath, currentPath));
                        ofInterest = false;     // not of interest now because it is in a container
                    }
                    else
                        currentBuffer = paths.get(currentPath.toString()).buffer;
                }
                pushAttributes(atts);           // in case any attributes are also of interest
            }
        }
    }


    private void copyTag(String qName, Attributes atts)
    {
        currentBuffer.append(String.format("<%s", qName));
        int len = atts.getLength();
        for (int i=0; i<len; i++)
        {
            currentBuffer.append(String.format(" %s='%s'", atts.getQName(i), atts.getValue(i)));
        }
        currentBuffer.append(">");
    }

    private void copyEndTag(String qName)
    {
        currentBuffer.append(String.format("</%s>", qName));
    }

    private boolean isOfInterest()
    {
        return paths.get(currentPath.toString()) != null;
    }

    // Save each attribute of interest in the buffer
    private void pushAttributes(Attributes atts)
    {
        int len = atts.getLength();
        int pathlen = currentPath.length();
        for (int i=0; i<len; i++)
        {
            currentPath.append(String.format(":%s", atts.getQName(i)));
            ElementInfo ei = paths.get(currentPath.toString());
            if (ei != null)                  // is this an attribute of interest?
                ei.buffer.append(atts.getValue(i));
            currentPath.setLength(pathlen);
        }
    }

    public void popElement(String qName)
    {
        // Do post processing, if any

        if (currentKey.toString().equals("TEI.teiHeader.fileDesc.sourceDesc.msDesc.history.origin.p"))
            debug = debug++ - 1;
        if (containerPath != null)          // are we currently in a container?
            copyEndTag(qName);

        ElementHandler eh = handlers.get(currentPath.toString());
        if (eh != null)
            eh.process(currentPath.toString());
        else if (ofInterest)
            defaultHandler.process(currentPath.toString());

        if (containerPath != null && containerPath.equals(currentPath.toString()))       // is this the end of the container?
            containerPath = null;

        // Pop the last tag off of the current key
        int n = currentKey.lastIndexOf(".");
        if (n != -1)
            currentKey.setLength(n);
        else
            currentKey.setLength(0);

        // Pop the last tag off of the current path
        n = currentPath.lastIndexOf(".");
        if (n != -1)
            currentPath.setLength(n);
        else
            currentPath.setLength(0);

        ofInterest = isOfInterest();
        if (ofInterest)
            currentBuffer = paths.get(currentPath.toString()).buffer;
        else if (containerPath == null)
            currentBuffer = null;
    }


    public String getTrimmedValue(String path)
    {
        ElementInfo ei = paths.get(path);
        if (ei == null)
            return String.format("%s not defined", path);
        else
        {
            String value = ei.buffer.toString();
            ei.buffer.setLength(0);
            return value.trim();
        }
    }


    public void clear(String path)
    {
        ElementInfo ei = paths.get(path);
        if (ei != null)
            ei.buffer.setLength(0);
    }

    public void append(String s)
    {
        if (containerPath != null || ofInterest)
            currentBuffer.append(s);
    }

    public void dump(String path)
    {
        System.out.print(path);
        System.out.print(": ");
        ElementInfo ei = paths.get(path);
        if (ei == null)
            System.out.println("null");
        else if (ei.buffer.length() == 0)
            System.out.println("none");
        else
           System.out.println(ei.buffer);
    }
}