/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package nsf;

import java.util.Hashtable;
import java.util.Enumeration;
import java.io.BufferedWriter;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.FonttblParser;
import org.sanskritlibrary.StylesheetParser;
import org.sanskritlibrary.PassThroughParser;
import org.sanskritlibrary.Rules;
import org.sanskritlibrary.ResourceInputStreamImpPath;
import org.sanskritlibrary.RTF2;
import org.sanskritlibrary.TextFormatter;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.Regex;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.RawEncoder;
/**
 *
 * @author ralph
 */
public class RTFParser extends RTFRoot
{
    String fileName;
    String computerType;
    String transcodersDirectory;

    public RTFParser(String fileName, String computerType, String transcodersDirectory)
    {
        this.fileName = fileName;
        this.computerType = computerType;
        this.transcodersDirectory = transcodersDirectory;
    }

    public void parse() throws Exception
    {
        parse(null);
    }

    public void parse(Hashtable<String, TransformMap> map) throws Exception
    {
        parseRTFFile(fileName, computerType, map);
        rtfFormat(fileName.replace(".rtf", "x.rtf"));
        rtfFormat(fileName);
    }

    public void parseRTFFile(String fileName, String platform, Hashtable<String, TransformMap> _map) throws Exception
    {
        Hashtable<String, TransformMap> map =  _map;
        
        try
        {
            FonttblParser fp = new FonttblParser();
            StylesheetParser sp = new StylesheetParser();
            PassThroughParser pt = new PassThroughParser();
            Rules rules = new Rules();
            if (map == null)        // use defaults
            {
                map = new Hashtable<String, TransformMap>();

                map.put("unitms", new TransformMap("*:cardonaroman->roman", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("lucidagrande", new TransformMap("*:cardonaroman->roman", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("times new roman", new TransformMap("*:cardonaroman->roman", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("maharegular", new TransformMap("*:cardonamahareg->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("devanagariextla", new TransformMap("*:cardonaext->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("maharishi1bold", new TransformMap("*:cardonambold->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("devanagarifix", new TransformMap("*:cardonadevfix->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("vedic", new TransformMap("*:vedic->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("vedic97", new TransformMap("*:vedic97->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("madhushree", new TransformMap("*:madhushree->deva", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
                map.put("manjushreecsx", new TransformMap("*:manjushreeCSX->roman", new ResourceInputStreamImpPath(transcodersDirectory), RTF2.fsmh));
            }

            // If transcoder XML file has a sharedState attribute, then make that the virtual font name so that several different fonts can work together, other make the virtual font name the same as the RTF font name
            Enumeration<String> en = map.keys();
            while (en.hasMoreElements())
            {
                String fontname = en.nextElement();
                TransformMap tm = map.get(fontname);
                tm.setVirtualFontname(fontname);
            }

            rules.add("fonttbl", fp);
            rules.add("stylesheet", sp);
            rules.add("colortbl", pt);
            rules.add("themedata", pt);
            rules.add("colorschememapping", pt);
            rules.add("latentstyles", pt);
            rules.add("datastyles", pt);
            rules.add("f", fp);
            rules.add("af", fp);
            rules.add("deff", fp);
            rules.add("adeff", fp);
            rules.add("dn", fp);
            parseRTF(fileName, rules, map, platform, "unitms");
        } catch (Exception ex)
        {
            System.out.println(RTF2.debug);
            throw ex;
        }

        if (FSM.unknownChars.size() > 0)
        {
            Enumeration<String> en = FSM.unknownChars.keys();
            while (en.hasMoreElements())
            {
                System.out.println("Unknown characters");
                String key = en.nextElement();
                int debug = FSM.unknownChars.get(key);
                System.out.format("%s %d\n", TextFormatter.charToSlashU(key), debug);
            }
        }

        // Display pass throughs for ManjushreeCSX
        TransformMap tm = map.get("manjushreecsx");
        if (tm != null)
        {
            FSM fsm = tm.getInitialFSM();
            System.out.println("\n\n************* Passthroughs");
            displayPassthroughs(fsm);
        }
    }

    public void parseRTF(String fileName, Rules rules,
            Hashtable<String, TransformMap> map,
            String platform, String defaultFont) throws Exception
    {
        String outFileName = Regex.replace(fileName, "\\.rtf$", "x.rtf");
        BufferedWriter bw = IOAssistance.openWriter(outFileName);
        RTF2 rtf = new RTF2(fileName, rules, map, bw, platform, defaultFont, null);
        String logFile = outFileName + ".log";
        BufferedWriter log = IOAssistance.openWriter(logFile);
        rtf.setLog(log);

        String rawFile = Regex.replace(fileName, "\\.rtf$", "x.raw");
        BufferedWriter raw = IOAssistance.openWriter(rawFile, new RawEncoder());
        rtf.setExtractionRaw(raw);

        String uniFile = Regex.replace(fileName, "\\.rtf$", "x.uni");
        BufferedWriter uni = IOAssistance.openWriter(uniFile);
        rtf.setExtractionProcessed(uni);

        System.out.format("Parsing %s\n", rtf.getFile().getName());
        rtf.parse();
        rtf.close();
        verify(rawFile, uniFile);
        dumpFrequencies();
    }
}
