package nsf;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;
import java.io.BufferedWriter;
import java.util.Collections;
import java.nio.charset.Charset;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.ResourceInputStreamImpPath;
import org.sanskritlibrary.RTF2;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.RawDecoder;
import org.sanskritlibrary.webservice.WebServices;

public class RTFRoot
{
    public void rtfFormat(String fileName) throws Exception
    {
        System.out.format("Format file is %sx\n", fileName);
        String s = IOAssistance.readEntireFile(fileName);
        BufferedWriter bw = IOAssistance.openWriter(fileName + "x");

        try
        {
            int n = rtfDump(s, 0, bw, 0);
            System.out.format("%d of %d bytes processed\n", n, s.length());
        } catch (Exception ex)
        {
            ex.printStackTrace();
        }
        bw.close();
    }

    public int rtfDump(String s, int n, BufferedWriter bw, int level) throws Exception
    {
        String margin = "\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t";
        bw.write("\n");         // start on a new line
        bw.write(margin.substring(0, level));       // indent proper amount
        bw.write(s.charAt(n++));                    // write the {
        while (!isBrace(s, n, '}'))
        {
            if (isBrace(s, n, '{'))                 // have nested group
            {
                n = rtfDump(s, n, bw, level + 1);
                bw.write(margin.substring(0, level + 1));     // indent same subgroup
            } else
            {
                if (s.charAt(n) == '\n' || s.charAt(n) == '\r')
                {
                    bw.write(' ');
                    n++;
                } else
                    bw.write(s.charAt(n++));
            }
        }

        bw.write('\n');                             // new line
        bw.write(margin.substring(0, level));
        if (n < s.length())
            bw.write(s.charAt(n++));                    // write the }
        return n;                                   // points after the }
    }

    public boolean isBrace(String s, int n, char brace)
    {
        if (n >= s.length())
            return true;
        
        if (s.charAt(n) != brace)
            return false;
        if (n > 1)      // make sure that a \ is not really a \\
        {
            if (s.charAt(n - 1) != '\\')
                return true;         // it is not preceded by \ so it must be the brace
            else if (s.charAt(n - 2) == '\\')
                return true;         // the \ is escaped by a \ so it must be the brace
            else
                return false;        // the \ is not escaped by a \ so we have \brace which is not a brace
        } else if (n > 0)     // n is equal to 1 here
            return s.charAt(n - 1) != '\\';  // if we have \brace return false, no way the \ can be escaped, otherwise return true
        else
            return true;    // brace is first character in s, hence can't be preceded by a \
    }

    public void verify(String rawFile, String uniFile) throws Exception
    {
        String raw = IOAssistance.readEntireFile(rawFile, new RawDecoder());
        TransformMap tm = new TransformMap("*:madhushree->deva",
                new ResourceInputStreamImpPath("C://Users//ralph//My Documents//PeterScharf//SanskritLibrary/data/transcoders"),
                RTF2.fsmh);

        String text2 = WebServices.transformString(raw, tm);
        String text1 = IOAssistance.readEntireFile(uniFile, "UTF-8");
        if (!text1.equals(text2))
        {
            boolean bad = false;
            int len1 = text1.length();
            int len2 = text2.length();
            int len = len1;
            if (len > len2)
                len = len2;
            for (int i = 0; i < len; i++)
            {
                if (text1.charAt(i) != text2.charAt(i))
                {
                    BufferedWriter bw = IOAssistance.openWriter(uniFile + "_1");
                    bw.write(text1.substring(0, i));
                    bw.write("****");
                    bw.write(text1.substring(i));
                    bw.close();

                    bw = IOAssistance.openWriter(uniFile + "_2");
                    bw.write(text2.substring(0, i));
                    bw.write("****");
                    bw.write(text2.substring(i));
                    bw.close();

                    System.out.println("******** descrepancy between first and second versions of unicode files");
                    break;
                }
            }
        }
    }

    protected void dumpFrequencies()
    {
        System.out.println("Dumping character frequencies");
        Enumeration<String> en = RTF2.usedChars.keys();
        ArrayList<String> a = new ArrayList<String>();
        while (en.hasMoreElements())
        {
            a.clear();
            String fontName = en.nextElement();
            Hashtable<String, Integer> h = RTF2.usedChars.get(fontName);
            Enumeration<String> en2 = h.keys();
            while (en2.hasMoreElements())
            {
                a.add(en2.nextElement());
            }

            Collections.sort(a);

            System.out.format("Characters used by %s", fontName);
            for (int i=0; i<a.size(); i++)
            {
                String key = a.get(i);
                System.out.format("%d. %s %d\n", Integer.parseInt(key.substring(2), 16), key, h.get(key));
            }
            System.out.println();
        }
    }

    static void displayPassthroughs(FSM fsm) throws Exception
    {
        Hashtable<Character, Integer> missing = fsm.getMissing();
        Enumeration<Character> en = missing.keys();
        ArrayList<String> arr = new ArrayList<String>();
        while (en.hasMoreElements())
        {
            Character ch = en.nextElement();
            int cnt = missing.get(ch);
            String s = FSM.toUnicodeString(ch.toString());
            if (s.length() == 1)
            {
                char chh = s.charAt(0);
                if (chh < ' ')
                {
                    String hex = Integer.toHexString(chh);
                    if (hex.length() == 1)
                    {
                        s = "\\u000";
                    } else
                    {
                        s = "\\u00";
                    }
                    s += hex;
                }
            }
            arr.add(String.format("%s: %d", s, cnt));
            //System.out.println(String.format("%s: %d", s, cnt));
        }
        Collections.sort(arr);

        for (String u : arr)
        {
            System.out.println(u);
        }
    }

}
