package nsf;

import java.io.BufferedWriter;
import java.util.Hashtable;
import java.util.ArrayList;
import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import org.sanskritlibrary.Regex;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.SAXParserFactory;
//import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import org.sanskritlibrary.IOAssistance;
import org.xml.sax.ext.DefaultHandler2;

public class TitusAnalysis extends DefaultHandler2 //implements ErrorHandler, ContentHandler, DTDHandler, EntityResolver
{
    StringBuilder sb;
    StringBuilder pathsb;
    String currentDirectory;
    String fn;
    Locator locator;
    Hashtable<String, ArrayList<ValueCount>> attributes;
    Hashtable<String, TEIInfo> structure;
    Hashtable<String, Hashtable<String, Boolean>> paths;
    SAXParser saxParser;
    ArrayList<String> stack;
    int lastdivno;
    int lastLineNumber;
    ArrayList<String> divstack;
    boolean inbody = false;
    int[] divcount = new int[10];
    boolean collectTitle = false;
    String titlePath = "TEI.teiHeader.fileDesc.titleStmt.title";
    StringBuilder titleValue = new StringBuilder();

    public TitusAnalysis()
    {
        sb = new StringBuilder();
        pathsb = new StringBuilder();
        stack = new ArrayList<String>();
        divstack = new ArrayList<String>();
        attributes = new Hashtable<String, ArrayList<ValueCount>>();
        structure = new Hashtable<String, TEIInfo>();
        paths = new Hashtable<String, Hashtable<String, Boolean>>();
        // Use the default (non-validating) parser
        SAXParserFactory factory = SAXParserFactory.newInstance();
        try
        {
            // Parse the input
            saxParser = factory.newSAXParser();
        }
        catch (Throwable t)
        {
            t.printStackTrace ();
        }
    }

    public boolean processFile(String filename)
    {
        try
        {
            fn = filename.substring(0, filename.length()-4);
            Regex re = new Regex(fn, "/([^/]+/[^/]+)$");
            if (re.find())
                fn = re.getGroup(1);
            lastdivno = 0;
            inbody = false;
            saxParser.parse( new File(filename), this);
            return true;
        }
        catch (Throwable t)
        {
            t.printStackTrace();
            return false;
        }
    }

    // For ErrorHandler
    @Override
    public void warning(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void comment (char buf[], int offset, int len) throws SAXException
    {
        System.out.println("found a comment");
    }

    @Override
    public void error(SAXParseException exception) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void fatalError(SAXParseException ex) throws SAXException
    {
        System.out.format("%s: %d.%d %s\n", ex.getSystemId(), ex.getLineNumber(), ex.getColumnNumber(), ex.getLocalizedMessage());
        throw ex;
    }

    // For ContentHandler
    @Override
    public void setDocumentLocator(Locator locator)
    {
        this.locator = locator;
    }

    @Override

    public void startDocument() throws SAXException
    {
    }

    @Override
    public void endDocument() throws SAXException
    {
        if (divstack.size() > 0)
        {
            this.handleDiv("div1", null);
            if (divstack.size() > 0)
                divstack.clear();
        }

        structure.get(fn).title = fixup(titleValue.toString());
        titleValue.setLength(0);
        TEIInfo tei = structure.get(fn);
        if (structure.get(fn).vc.size() == 1)
        {
            ValueCount vc = tei.vc.get(0);
            for (int i=0; i<divcount.length; i++)
            {
                vc.divcount[i] = divcount[i];
            }
        }
        for (int i=0; i<divcount.length; i++)
        {
            tei.divcount[i] = divcount[i];
            divcount[i] = 0;
        }
    }

    @Override
    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void endPrefixMapping(String prefix) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException
    {
        if (qName.equals("body"))
            inbody = true;

        stack.add(qName);
        if (pathgen().equals(titlePath))
            collectTitle = true;

        String pk = pathgen();
        Hashtable<String, Boolean> h = paths.get(pk);
        if (h == null)
        {
            h = new Hashtable<String, Boolean>();
            paths.put(pk, h);
        }

        h.put(fn, Boolean.TRUE);

        int numatts = atts.getLength();
        for (int i=0; i<numatts; i++)
        {
            String attname = atts.getQName(i);
            String value = atts.getValue(i);
            String key = String.format("%s/%s", qName, attname);
            ArrayList<ValueCount> a = attributes.get(key);
            if (a == null)
            {
                a = new ArrayList<ValueCount>();
                attributes.put(key, a);
            }

            ValueCount valueCount = null;
            for (ValueCount vc : a)
            {
                if (vc.value.equals(value))
                {
                    valueCount = vc;
                    break;
                }
            }

            if (valueCount == null)
                a.add(valueCount = new ValueCount(value));

            valueCount.count++;

            valueCount.h.put(fn, Boolean.TRUE);
        }

        if (inbody && (qName.equals("div1") || qName.equals("div2") || qName.equals("div3") || qName.equals("div4") || qName.equals("div5") || qName.equals("div6")))
            handleDiv(qName, atts.getValue("type"));
    }

    private void handleDiv(String qName, String value)
    {
        int divno = Integer.parseInt(qName.substring(3));

        if (value != null)
        {
            if (divno < divcount.length)
                divcount[divno]++;

            lastLineNumber = locator.getLineNumber();
        }
        if (divno <= lastdivno)
        {
            sb.setLength(0);
            String delim = "";
            for (String div : divstack)
            {
                sb.append(String.format("%s%s", delim, div));
                delim = ".";
            }
            TEIInfo tei = structure.get(fn);
            if (tei == null)
            {
                tei = new TEIInfo();
                structure.put(fn, tei);
            }

            String skey = sb.toString();
            ValueCount foundvc = null;
            for (ValueCount vc : tei.vc)
            {
                if (vc.value.equals(skey))
                {
                    foundvc = vc;
                    break;
                }
            }

            if (foundvc == null)
            {
                foundvc = new ValueCount(skey);
                if (value != null)
                    foundvc.lineno = locator.getLineNumber();
                else
                    foundvc.lineno = lastLineNumber;
                tei.vc.add(foundvc);
            }
            foundvc.count++;

            for (int i = 0; i <= lastdivno - divno; i++)
            {
                divstack.remove(divstack.size() - 1);
            }
        }
        if (value != null)
            divstack.add(value);

        lastdivno = divno;
    }

    private String pathgen()
    {
        pathsb.setLength(0);
        String delim = "";
        for (String e : stack)
        {
            pathsb.append(String.format("%s%s", delim, e));
            delim = ".";
        }

        return pathsb.toString();
    }

    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        if (pathgen().equals(titlePath))
            collectTitle = false;
        else if (collectTitle && top().equals("foreign"))
            titleValue.append("%");
        stack.remove(stack.size()-1);
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException
    {
        if (collectTitle)
        {
            if (top().equals("foreign"))
                titleValue.append("%");
            titleValue.append(new String(ch, start, length));
        }
    }

    private String top()
    {
        return stack.get(stack.size()-1);
    }

    @Override
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void processingInstruction(String target, String data) throws SAXException
    {
    }

    @Override
    public void skippedEntity(String name) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    // For DTDHandler
    @Override
    public void notationDecl(String name, String publicId, String systemId) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void unparsedEntityDecl(String name, String publicId, String systemId, String notationName) throws SAXException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    // For EntityHandler
    @Override
    public InputSource resolveEntity(String publicId, String systemId) throws SAXException, IOException
    {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    public void produceAttributeReport(String filename) throws Exception
    {
        Enumeration<String> e = attributes.keys();
        ArrayList<String> a = new ArrayList();
        while (e.hasMoreElements())
        {
            a.add(e.nextElement());
        }

        Collections.sort(a);

        BufferedWriter bw = IOAssistance.openWriter(filename);
        for (String k : a)
        {
            bw.write(String.format("%s\n", k));
        }

        bw.write("\n\n");

        for (String k : a)
        {
            bw.write(String.format("%s\n", k));
            ArrayList<ValueCount> al = attributes.get(k);
            for (ValueCount vc : al)
            {
                bw.write(String.format("\t%s [%d] - (%s)\n", vc.value, vc.count, vc.getFiles()));
            }
            bw.write("---------------\n");
        }

        bw.close();
    }

    public void producePathReport(String filename) throws Exception
    {
        Enumeration<String> e = paths.keys();
        ArrayList<String> a = new ArrayList();
        while (e.hasMoreElements())
        {
            a.add(e.nextElement());
        }

        Collections.sort(a);

        BufferedWriter bw = IOAssistance.openWriter(filename);
        for (String k : a)
        {
            Hashtable<String, Boolean> h = paths.get(k);
            sb.setLength(0);
            e = h.keys();
            String delim = "";
            while (e.hasMoreElements())
            {
                sb.append(String.format("%s%s", delim, e.nextElement()));
                delim = ",";
            }
            bw.write(String.format("%s .......(%s)\n", k, sb.toString()));
        }

        bw.close();
    }

    public void produceMenuReport(String filename) throws Exception
    {
        Enumeration<String> e = structure.keys();
        ArrayList<String> a = new ArrayList();
        while (e.hasMoreElements())
        {
            a.add(e.nextElement());
        }

        Collections.sort(a);

        BufferedWriter bw = IOAssistance.openWriter(filename);
        bw.write(String.format("<texts>\n"));
        int okcount = 0;
        int workcount = 0;
        for (int pass = 0; pass < 2; pass++)
        {
            for (String k : a)
            {
                TEIInfo tei = structure.get(k);
                ArrayList<ValueCount> avc = tei.vc;
                if (pass == 1 && avc.size() >= 2)
                {
                    String maxpath = "";
                    if (avc.size() >= 2)
                    {
                        workcount++;
                        bw.write(String.format("\t<text name='%s' title='%s'>\n", k, tei.title));
                        for (ValueCount vc : avc)
                        {
                            if (vc.value.length() > maxpath.length())
                                maxpath = vc.value;
                            bw.write(String.format("\t\t<path count='%d' first='%d'>%s</path>\n", vc.count, vc.lineno - 1, vc.value));
                        }
                    }

                    if (avc.size() == 2)
                    {
                        boolean prefix = true;
                        for (ValueCount vc : avc)
                        {
                            if (!maxpath.startsWith(vc.value))
                            {
                                prefix = false;
                                break;
                            }
                        }

                        if (prefix)
                        {
                            String[] comp = maxpath.split("\\.");
                            for (int i=1; i<=comp.length; i++)
                            {
                                int dc = tei.divcount[i];
                                if (dc == 0)
                                    break;
                                String comment = null;
                                if (i == 1)
                                    comment = String.format("%d %ss", dc, comp[i-1]);
                                else
                                {
                                    int dc0 = tei.divcount[i-1];
                                    double ratio = (double)(dc)/(double)dc0;
                                    comment = String.format("%1.1f %ss per %s", ratio, comp[i-1], comp[i-2]);
                                }
                                bw.write(String.format("\t\t<div name='%s' size='%d'/>   <!-- %s -->\n", comp[i-1], dc, comment));
                            }
                        }
                    }

                    bw.write(String.format("\t</text>\n\n", k));
                }
                else if (pass==0 && avc.size() == 1)      // menu stack is well-defined
                {
                    okcount++;
                    bw.write(String.format("\t<text name='%s' title='%s'>\n", k, structure.get(k).title));
                    ValueCount vc = avc.get(0);
                    bw.write(String.format("\t\t<path count='%d' first='%d'>\n", vc.count, vc.lineno - 1));
                    bw.write(String.format("\t\t\t<value>%s</value>\n", vc.value));
                    String[] menu = vc.value.split("\\.");
                    double div = 1.0;
                    for (int i = 0; i < menu.length; i++)
                    {
                        bw.write(String.format("\t\t\t<menu name='%s' size='%1.1f'/>\n", menu[i], vc.divcount[i + 1] / div));
                        div = vc.divcount[i + 1];
                    }
                    bw.write(String.format("\t\t</path>\n"));
                    bw.write(String.format("\t</text>\n\n", k));
                }

            }

            if (pass == 0)
                bw.write("\n<!--- These have multiple possible menu stacks --->\n\n");
        }

        System.out.println(String.format("ok=%d, need work=%d\n", okcount, workcount));

        bw.write(String.format("</texts>\n"));

        bw.close();

    }

    private String fixup(String s)
    {
        return s.replaceAll("\\s+", " ");
    }
}
