package nsf;

import java.io.BufferedReader;
import java.util.Hashtable;
import java.util.ArrayList;
import java.util.Enumeration;

import org.sanskritlibrary.webservice.RESTfulWSCaller;
import org.sanskritlibrary.webservice.WebServices;
import org.sanskritlibrary.IOAssistance;
import org.sanskritlibrary.ResourceInputStream;
import org.sanskritlibrary.webservice.TransformMap;
import org.sanskritlibrary.FSM;
import org.sanskritlibrary.Regex;

public class WordLookup
{
    int numErrors = 0;
    String directory;
    String[] files;
    ResourceInputStream ris;
    Hashtable<String, FSM[]> fmsh = new Hashtable<String, FSM[]>();
    TransformMap tm;

    public WordLookup(ResourceInputStream ris, Hashtable<String, FSM[]> fmsh) throws Exception
    {
        this.ris = ris;
        this.fmsh = fmsh;
        this.tm = new TransformMap("*:roman->slp1", ris, fmsh);
    }

    public ArrayList<CompareInfo> mergeFiles(String directory, String[] filenames) throws Exception
    {
        Hashtable<String, CompareInfo> h = new Hashtable<String, CompareInfo>();
        ArrayList<CompareInfo> a =  new ArrayList<CompareInfo>();
        for (String filename : filenames)
        {
            BufferedReader br = IOAssistance.openReader(directory+filename);
            String line;
            Regex re = new Regex("", "^(\\d+)\\s*<a>(.*)</a>");
            while ((line=br.readLine()) != null)
            {
                re.reset(line);

                if (re.find())
                {
                    String slp1 = WebServices.transformString(re.getGroup(2), tm);
                    CompareInfo cmpi = h.get(slp1);
                    if (cmpi == null)
                    {
                        cmpi = new CompareInfo(slp1);
                        h.put(slp1, cmpi);
                    }
                    cmpi.incrementWordCount(Integer.parseInt(re.getGroup(1)));
                    cmpi.addFileName(filename);
                }
            }
        }
        
        Enumeration<String> e = h.keys();
        while(e.hasMoreElements())
        {
            String key = e.nextElement();
            a.add(h.get(key));
        }

        return a;
    }


    public void compareX(ArrayList<CompareInfo> a)
    {
        for (CompareInfo ci : a)
        {
            int n = (int)(Math.random() * 4);
            switch (n)
            {
                case 0:
                    ci.gerardFound = false;
                    ci.jimFound = false;
                    break;
                case 1:
                    ci.gerardFound = false;
                    ci.jimFound = true;
                    break;
                case 2:
                    ci.gerardFound = true;
                    ci.jimFound = false;
                    break;
                case 3:
                    ci.gerardFound = true;
                    ci.jimFound = true;
                    break;
            }
        }
    }

    
    public void compare(CompareInfo ci) throws Exception
    {
        compareGerard(ci);
        compareJim(ci);
    }

    public void analyze(CompareInfo ci, String filename) throws Exception
    {
        StringBuilder sb = new StringBuilder();

        String line = String.format("%s%s   %s\t",
                ci.gerardFound ? "G" : ".",
                ci.jimFound ? "J" : ".",
                ci.word);
        sb.append(line);
        String delim = "";
        for (String s : ci.files)
        {
            sb.append(delim);
            sb.append(s);
            delim=",";
        }
        sb.append("\n");

        IOAssistance.append(filename, sb.toString());
    }

    // http://sanskrit.inria.fr/cgi-bin/sktreader?t=SL&topic=&mode=p&text=aanunAsikO
    public void compareGerard(CompareInfo ci) throws Exception
    {
        int n = 8040;
        int lineno = 0;
        String uri = "http://sanskrit.inria.fr/cgi-bin/sktreader";
        Hashtable h = new Hashtable();
        h.put("t", "SL");
        h.put("topic", "");
        h.put("mode", "p");

        try
        {
            lineno++;
            h.put("text", ci.word);
            String result = RESTfulWSCaller.callText(uri, h);
            ci.gerardFound = result.indexOf("No solution to chunk") == -1;
        }
        catch (Exception ex)
        {
            reportError(ci, "compareGerard", ex);
        }
    }

    // http://www.language.brown.edu/Sanskrit/Funderburk/fflex/web/rp1/getWord.php?filterin=SLP2SLP&filter=SLP2SLP&word=aanunAsikO
    // http://www.language.brown.edu/Sanskrit/Funderburk/fflex/web/rp1/getWord.php?filterin=SLP2SLP&filter=SLP2SLP&word=aanunAsikO
    public void compareJim(CompareInfo ci) throws Exception
    {
        int lineno = 0;
        String uri = "http://www.language.brown.edu//Sanskrit/Funderburk/fflex/web/rp1/getWord.php";
        Hashtable h = new Hashtable();
        h.put("filterin", "SLP2SLP");
        h.put("filter", "SLP2SLP");
        try
        {
            lineno++;
            h.put("word", ci.word);
            String result = RESTfulWSCaller.callText(uri, h);
            ci.jimFound = result.indexOf("no data found") == -1;
        }
        catch (Exception ex)
        {
            reportError(ci, "compareJim", ex);
        }
    }

    private void reportError(CompareInfo ci, String source, Exception ex) throws Exception
    {
        numErrors++;
        String msg = String.format("%d. In %s: %s: %s\n", ci.lineno, source, ci.word, ex.getMessage());
        IOAssistance.append("errors.txt", msg);
        System.out.print(msg);
        if (numErrors > 100)
            throw new Exception("Too many errors");
    }
}
